from cmath import exp

__author__ = 'Mikael'
# -*- coding: UTF-8 -*-


import sys, getopt
from os.path import isfile, join, exists, isdir
from os import makedirs


def printUsage():
    print 'ExperimentationBuilder.py -i <pathToTheQueryFiles> [-n <numberOfQueriesPerFile>'


def writeQueriesToFile(file, listQueries, listSize, nbQueries):

    nbAddedQueries = 0
    while nbAddedQueries < nbQueries:
        remainingNbQueries = nbQueries - nbAddedQueries
        if listSize <= remainingNbQueries:
            for query in listQueries:
                file.write(query)
            nbAddedQueries += listSize
        else:
            for query in listQueries[0:remainingNbQueries]:
                file.write(query)
            nbAddedQueries += remainingNbQueries


def buildExperimentation(listQueries, cacheMissQueries, nbQueries, outputPath, expFileNameBase):

    lengthListQueries = len(listQueries)
    lengthCacheMissQueries = len(cacheMissQueries)

    for percent in range(0,101,10):
        curExpFileName = expFileNameBase + str(percent) + '.txt'
        curExpFilePath = join(outputPath,curExpFileName)
        file = open(curExpFilePath,"w")

        #write the experimentation queries
        writeQueriesToFile(file, listQueries, lengthListQueries, nbQueries*percent/100)

        #write the cache miss queries to complete
        writeQueriesToFile(file, cacheMissQueries, lengthCacheMissQueries, nbQueries*(100-percent)/100)

        file.close()


def isValid(path):
    return exists(path) and isfile(path)

def loadQueriesFromFiles(exactHitPath, extendedHitPath, partialHitPath, missPath):
    exactHitFile = open(exactHitPath,'r')
    extendedHitFile = open(extendedHitPath,'r')
    partialHitFile = open(partialHitPath,'r')
    missFile = open(missPath,'r')

    listExactHit = [query for query in exactHitFile]
    listExtendedHit = [query for query in extendedHitFile]
    listPartialHit = [query for query in partialHitFile]
    listMiss = [query for query in missFile]

    exactHitFile.close()
    extendedHitFile.close()
    partialHitFile.close()
    missFile.close()

    return listExactHit, listExtendedHit, listPartialHit, listMiss

def getPredicateRelationPercentage(attribute,min,max):

    min = int(min)
    max = int(max)

    dictAttributeSize = {
        'id':200000.0,
        'intattr1':10000.0,
        'intattr2':50000.0,
        'intattr3':100000.0
    }

    if max == 0:
        max = dictAttributeSize[attribute]

    if (max < min):
        raise ValueError("getPredicateRelationPercentage: min cannot be superior to max")

    return (max-min)/dictAttributeSize[attribute]

def getQuerySize(query):
    queryPercentage = 0
    predicates = query.split("WHERE")[1].split("AND")
    if len(predicates) == 0:
        print 'error in query list'
    elif len(predicates) == 1:
        curPredicate = predicates[0].strip()
        if "<" in curPredicate:
            predicateItems = curPredicate.split("<")
            queryPercentage = getPredicateRelationPercentage(predicateItems[0].strip(),0,predicateItems[1].strip(' ;='))
        elif ">" in curPredicate:
            predicateItems = curPredicate.split(">")
            queryPercentage = getPredicateRelationPercentage(predicateItems[0].strip(),predicateItems[1].strip(' ;='),0)
        elif " = " in curPredicate:
            predicateItems = curPredicate.split("=")
            queryPercentage = getPredicateRelationPercentage(predicateItems[0].strip(),0,1)
        else:
            raise ValueError("Some operators in the query file are not correct")

    if len(predicates) == 2:
        min = 0
        max = 0

        #handle first predicate
        curPredicate = predicates[0].strip()
        predicateAttribute1 = ""
        if "<" in curPredicate:
            predicateItems = curPredicate.split("<")
            predicateAttribute1 = predicateItems[0].strip()
            max = predicateItems[1].strip(' ;=')
        elif ">" in curPredicate:
            predicateItems = curPredicate.split(">")
            predicateAttribute1 = predicateItems[0].strip()
            min = predicateItems[1].strip(' ;=')
        elif " = " in curPredicate:
            raise ValueError("The generation of the query sets have failes")
        else:
            raise ValueError("Some operators in the query file are not correct")

        #handle second predicate
        curPredicate = predicates[1].strip()
        predicateAttribute2 = ""
        if "<" in curPredicate:
            predicateItems = curPredicate.split("<")
            predicateAttribute2 = predicateItems[0].strip()
            max = predicateItems[1].strip(' ;=')
        elif ">" in curPredicate:
            predicateItems = curPredicate.split(">")
            predicateAttribute2 = predicateItems[0].strip()
            min = predicateItems[1].strip(' ;=')
        elif " = " in curPredicate:
            raise ValueError("The generation of the query sets have failed")
        else:
            raise ValueError("Some operators in the query file are not correct")

        if min == 0 and max == 0:
            raise ValueError("The max and min are equal to 0, error in generation of queries")
        elif predicateAttribute1 != predicateAttribute2:
            raise ValueError("The two predicate do not have the same attribute, error in generation of queries")
        else:
            queryPercentage = getPredicateRelationPercentage(predicateItems[0].strip(),min,max)

    return queryPercentage

def getAvgQuerySizeFromExpFile(file,nbRead):

    totalQueryPercentage = 0.0
    querySizes = []

    lines = [query for query in file]

    for i in range(0,nbRead):
        curQuerySize = getQuerySize(lines[i])
        querySizes.append(curQuerySize*200000)

    print querySizes

    totalQuerySize = sum(querySizes)

    #result = totalQuerySize/len(querySizes)*100 if len(querySizes) > 0 else 0
    result = totalQuerySize

    return result

def printAvgQuerySizeGraph(pathExperimentation, experimentationFilesBase):

    expAvgQuerySizes = []

    nbRead = 0
    for percent in range(0,101,10):

        curExpFileName = experimentationFilesBase + str(percent) + '.txt'
        curExpFilePath = join(pathExperimentation,curExpFileName)
        file = open(curExpFilePath,"r")

        curAvgQuerySize = getAvgQuerySizeFromExpFile(file,nbRead)
        nbRead+=5

        expAvgQuerySizes.append(curAvgQuerySize)

        file.close()

    print expAvgQuerySizes

def main():
    cacheExactHitQueriesFileName = 'cache_exact_hit_queries.txt'
    cacheExtendedHitQueriesFileName = 'cache_extended_hit_queries.txt'
    cachePartialHitQueriesFileName = 'cache_partial_hit_queries.txt'
    cacheMissQueriesFileName = 'cache_miss_queries.txt'

    inputFilesPath = ''
    outputPath = ''

    numberOfQueries = 50

    try:
        opts, args = getopt.getopt(sys.argv[1:], "hi:n:", ["help","ipath=","nbq="])
    except getopt.GetoptError:
        printUsage()
        return 1

    for opt, arg in opts:
        if opt == '-h':
            printUsage()
            return 0
        elif opt in ('-i','--ipath'):
            inputFilesPath = arg.decode("ISO-8859-15")
        elif opt in ('-n', '--nbq'):
            tmpArg = int(arg)
            if tmpArg < 0:
                return 2
            else:
                numberOfQueries = tmpArg
        else:
            print 'invalid option or no option specified'
            printUsage()

    if not inputFilesPath:
        print 'the location of the query files must be specified'
        return 3

    if not exists(inputFilesPath) and not isdir(inputFilesPath):
        print 'the location of the query files must exist and be a directory'
        return 4


    cacheExactHitQueriesFilePath = join(inputFilesPath, cacheExactHitQueriesFileName)
    cacheExtendedHitFilePath = join(inputFilesPath, cacheExtendedHitQueriesFileName)
    cachePartialHitFilePath = join(inputFilesPath, cachePartialHitQueriesFileName)
    cacheMissFilePath = join(inputFilesPath, cacheMissQueriesFileName)

    print cacheExactHitQueriesFilePath
    print cacheExtendedHitFilePath
    print cachePartialHitFilePath
    print cacheMissFilePath


    if not isValid(cacheExactHitQueriesFilePath) or not isValid(cacheExtendedHitFilePath) or not isValid(cachePartialHitFilePath) or not isValid(cacheMissFilePath):
        print 'some files from {0}, {1}, {2} or {3} are missing, please you specified the right location and try again'.format(cacheExactHitQueriesFileName, cacheExtendedHitQueriesFileName, cachePartialHitQueriesFileName, cacheMissQueriesFileName)
        return 5

    listQueriesExactHit, listQueriesExtendedHit, listQueriesPartialHit, listQueriesMiss = loadQueriesFromFiles(cacheExactHitQueriesFilePath, cacheExtendedHitFilePath, cachePartialHitFilePath, cacheMissFilePath)

    outputPath = join(inputFilesPath, 'EXP_FILES')

    if not exists(outputPath):
        makedirs(outputPath)

    outputPathExactHit = join(outputPath,'EXP_EXACT_HIT')
    outputPathExtendedHit = join(outputPath,'EXP_EXTENDED_HIT')
    outputPathPartialHit = join(outputPath,'EXP_PARTIAL_HIT')

    if not exists(outputPathExactHit):
        makedirs(outputPathExactHit)

    if not exists(outputPathExtendedHit):
        makedirs(outputPathExtendedHit)

    if not exists(outputPathPartialHit):
        makedirs(outputPathPartialHit)

    outputFileBaseExactHit = 'queries_exp_exact_hit_'
    outputFileBaseExtendedHit = 'queries_exp_extended_hit_'
    outputFileBasePartialHit = 'queries_exp_partial_hit_'


    buildExperimentation(listQueriesExactHit,listQueriesMiss,numberOfQueries,outputPathExactHit,outputFileBaseExactHit)
    buildExperimentation(listQueriesExtendedHit, listQueriesMiss, numberOfQueries, outputPathExtendedHit,outputFileBaseExtendedHit)
    buildExperimentation(listQueriesPartialHit, listQueriesMiss, numberOfQueries, outputPathPartialHit,outputFileBasePartialHit)

    #printAvgQuerySizeGraph(outputPathPartialHit,outputFileBasePartialHit)

    return 0


if __name__ == "__main__":
    sys.exit(main())