__author__ = 'Mikael'

from QueryMeasurement import QueryMeasurement

class Experimentation:

    def __init__(self):
        self.nbPosedQueries = 0
        self.nbFailedQueries = 0
        self.nbProcessedQueries = 0
        self.nbQueryCacheExactHit = 0
        self.nbQueryCacheMiss = 0
        self.nbQueryCacheReplacement = 0
        self.nbQueryCacheExtendedHit = 0
        self.nbQueryCachePartialHit = 0
        self.nbMobileEstimationCacheHit = 0
        self.nbMobileEstimationCacheMiss = 0
        self.nbCloudEstimationCacheHit = 0
        self.nbCloudEstimationCacheMiss = 0
        self.nbQueriesProcessedOnCloud = 0
        self.nbQueriesProcessedOnMobile = 0
        self.queryMeasurements = []

    def __str__(self):
        str_list = []
        str_list.append("Number of posed queries: " + str(self.nbPosedQueries))
        str_list.append("Number of failed queries: " + str(self.nbFailedQueries))
        str_list.append("Number of processed queries: " + str(self.nbProcessedQueries))
        str_list.append("Number of exact hit on query cache: " + str(self.nbQueryCacheExactHit))
        str_list.append("Number of miss on query cache: " + str(self.nbQueryCacheMiss))
        str_list.append("Number of query cache replacements: " + str(self.nbQueryCacheReplacement))
        str_list.append("Number of extended on query cache: " + str(self.nbQueryCacheExtendedHit))
        str_list.append("Number of partial hit on query cache: " + str(self.nbQueryCachePartialHit))
        str_list.append("Number of mobile estimation cache hit: " + str(self.nbMobileEstimationCacheHit))
        str_list.append("Number of cloud estimation cache hit: " + str(self.nbCloudEstimationCacheHit))
        str_list.append("Number of mobile estimation cache miss: " + str(self.nbMobileEstimationCacheMiss))
        str_list.append("Number of cloud estimation cache miss: " + str(self.nbCloudEstimationCacheMiss))
        str_list.append("Number of queries processed on cloud: " + str(self.nbQueriesProcessedOnCloud))
        str_list.append("Number of queries processed on mobile: " + str(self.nbQueriesProcessedOnMobile))
        str_list.append("Total Processing Time: " + str(self.getTotalProcessingTime()))
        str_list.append("Total Energy Consumption: " + str(self.getTotalEnergyConsumption()))
        str_list.append("Total Money Cost: " + str(self.getTotalMoneyCost()))
        str_list.append("Average Processing Time: " + str(self.getAvgProcessingTime()))
        str_list.append("Average Energy Consumption: " + str(self.getAvgEnergyConsumption()))
        str_list.append("Average Money Cost: " + str(self.getAvgMoneyCost()))
        for m in self.queryMeasurements:
            str_list.append(str(m))

        return ''.join([item + '\n' for item in str_list])


    def addQueryMeasurement(self,queryMeasurement):
        self.queryMeasurements.append(queryMeasurement)

    def addPosedQuery(self):
        self.nbPosedQueries += 1

    def addFailedQuery(self):
        self.nbFailedQueries += 1

    def addProcessedQuery(self):
        self.nbProcessedQueries += 1

    def addQueryCacheExactHit(self):
        self.nbQueryCacheExactHit += 1

    def addQueryCacheExtendedHit(self):
        self.nbQueryCacheExtendedHit += 1

    def addQueryCachePartialHit(self):
        self.nbQueryCachePartialHit += 1

    def addQueryCacheMiss(self):
        self.nbQueryCacheMiss += 1

    def addQueryCacheReplacement(self):
        self.nbQueryCacheReplacement += 1

    def addMobileEstimationCacheHit(self):
        self.nbMobileEstimationCacheHit += 1

    def addMobileEstimationCacheMiss(self):
        self.nbMobileEstimationCacheMiss += 1

    def addCloudEstimationCacheHit(self):
        self.nbCloudEstimationCacheHit += 1

    def addCloudEstimationCacheMiss(self):
        self.nbCloudEstimationCacheMiss += 1

    def addQueryProcessedOnCloud(self):
        self.nbQueriesProcessedOnCloud += 1

    def addQueryProcessedOnMobile(self):
        self.nbQueriesProcessedOnMobile += 1

    def getAdaptedTotalProcessingTime(self):
        return sum([long(mes.getAdaptedProcessingTime()) for mes in self.queryMeasurements])

    def getAdaptedTotalProcessingEnergy(self):
        return sum([long(mes.getAdaptedProcessingEnergy()) for mes in self.queryMeasurements])

    def getCloudProcessingTime(self):
        return sum([long(mes.getCloudProcessing().time) for mes in self.queryMeasurements])

    def getNbQueriesNotRespectingConstraints(self):
        return len([mes.getCloudProcessing().money for mes in self.queryMeasurements if mes.getCloudProcessing().money > 0.015])

    def setDiffCloudProcessingTime(self,otherQueryMeasurements):
        for i in range(0,len(self.queryMeasurements)):
            self.queryMeasurements[i].setDiffCloudProcessingTime(self.queryMeasurements[i].getCloudProcessing().time - otherQueryMeasurements[i].getCloudProcessing().time)

    def getMobileProcessingTime(self):
        return sum([long(mes.getMobileProcessing().time) for mes in self.queryMeasurements])

    def getTotalProcessingTime(self):
        #return sum([mes.getProcessing().time - mes.getDiffCloudProcessingTime() for mes in self.queryMeasurements])
        return sum([mes.getProcessing().time for mes in self.queryMeasurements])

    def getTotalEnergyConsumption(self):
        return sum([float(mes.getProcessing().energy)for mes in self.queryMeasurements])

    def getTotalMoneyCost(self):
        return sum([float(mes.getCloudProcessing().money) + float(mes.getComputeCloudEstimation().money) for mes in self.queryMeasurements])

    def getAvgProcessingTime(self):
        l = [long(mes.getProcessing().time) for mes in self.queryMeasurements]
        return sum(l)/len(l)

    def getAvgEnergyConsumption(self):
        l = [float(mes.getProcessing().energy) for mes in self.queryMeasurements]
        return sum(l)/len(l)

    def getAvgMoneyCost(self):
        l = [float(mes.getCloudProcessing().money) + float(mes.getComputeCloudEstimation().money) for mes in self.queryMeasurements]
        return sum(l)/len(l)

    def getTotalDownloadData(self):
        return sum([long(mes.getDownloadData().time)/5 for mes in self.queryMeasurements])

    def getTotalCacheAnalysis(self):
        return sum([long(mes.getCacheAnalysis().time) for mes in self.queryMeasurements])

    def getTotalQueryExecution(self):
        return sum([long(mes.getQueryExecution().time) for mes in self.queryMeasurements])

    def getTotalSizeOfQueriesReprocessedOnCloud(self):
        result = int(0)
        for mes in self.queryMeasurements:
            if mes.isReprocessedOnCloud():
                result += mes.getDownloadedResultSize()

        return result

    def getTotalSizeOfQueriesProcessedOnCloud(self):
        return sum([long(mes.getDownloadedResultSize()) for mes in self.queryMeasurements])