__author__ = 'Mikael'

from Experimentation import Experimentation
from QueryMeasurement import QueryMeasurement
from QueryMeasurement import Cost
from collections import namedtuple
from HitType import HitType

class ExperimentationParser(object):

    singleEventTag = ["QUERY_CACHE_EXACT_HIT","QUERY_CACHE_EXTENDED_HIT","QUERY_CACHE_PARTIAL_HIT",
                      "QUERY_CACHE_MISS","MOBILE_ESTIMATION_CACHE_HIT","CLOUD_ESTIMATION_CACHE_HIT",
                      "MOBILE_ESTIMATION_CACHE_MISS","CLOUD_ESTIMATION_CACHE_MISS","QUERY_PROCESS_ON_CLOUD",
                      "QUERY_PROCESS_ON_MOBILE","POSED_QUERY","PROCESSED_QUERY","FAILED_QUERY","CLOUD_PROCESS_RESULT_SIZE", "QUERY_CACHE_REPLACEMENT"]

    timeLapseTags = ["START_COMPUTE_CLOUD_ESTIMATION","STOP_COMPUTE_CLOUD_ESTIMATION", "START_MOBILE_PROCESS",
                     "STOP_MOBILE_PROCESS", "START_CLOUD_PROCESS","STOP_CLOUD_PROCESS","START_DOWNLOAD_DATA",
                     "STOP_DOWNLOAD_DATA", "START_DOWNLOAD_ESTIMATION_RESULT", "STOP_DOWNLOAD_ESTIMATION_RESULT",
                     "START_QUERY_PROCESS", "STOP_QUERY_PROCESS", "START_CACHE_ANALYSIS", "STOP_CACHE_ANALYSIS",
                     "START_QUERY_EXECUTION", "STOP_QUERY_EXECUTION", "START_DECISION_PROCESS", "STOP_DECISION_PROCESS",
                     "START_CACHE_REPLACEMENT", "STOP_CACHE_REPLACEMENT"]

    @staticmethod
    def parseLine(line):
        valueTag = namedtuple('ValueTag',['tag','value','extra_arg'])
        tagList = line.split(':')
        if len(tagList) >= 2:
            valueTag.tag = tagList[0]
            valueTag.value = tagList[1]
            if (len(tagList) == 3):
                valueTag.extra_arg = tagList[2]

        return valueTag

    @staticmethod
    def execSingleTagFunc(exp, tag):
        singleTagFuncMap = {
            "QUERY_CACHE_EXACT_HIT": exp.addQueryCacheExactHit,
            "QUERY_CACHE_EXTENDED_HIT": exp.addQueryCacheExtendedHit,
            "QUERY_CACHE_PARTIAL_HIT": exp.addQueryCachePartialHit,
            "QUERY_CACHE_MISS": exp.addQueryCacheMiss,
            "MOBILE_ESTIMATION_CACHE_HIT": exp.addMobileEstimationCacheHit,
            "CLOUD_ESTIMATION_CACHE_HIT": exp.addCloudEstimationCacheHit,
            "MOBILE_ESTIMATION_CACHE_MISS": exp.addMobileEstimationCacheMiss,
            "CLOUD_ESTIMATION_CACHE_MISS": exp.addCloudEstimationCacheMiss,
            "QUERY_PROCESS_ON_CLOUD": exp.addQueryProcessedOnCloud,
            "QUERY_PROCESS_ON_MOBILE": exp.addQueryProcessedOnMobile,
            "POSED_QUERY": exp.addPosedQuery,
            "PROCESSED_QUERY": exp.addProcessedQuery,
            "FAILED_QUERY": exp.addFailedQuery,
            "QUERY_CACHE_REPLACEMENT":exp.addQueryCacheReplacement
        }

        singleTagFuncMap[tag]()


    @staticmethod
    def execTimeLapseTagFunc(queryMeasurement, tag, currentCost):

        timeLapseSetterMap = {
            "STOP_COMPUTE_CLOUD_ESTIMATION":queryMeasurement.setComputeCloudEstimationCost,
            "STOP_MOBILE_PROCESS":queryMeasurement.setMobileProcessingCost,
            "STOP_CLOUD_PROCESS":queryMeasurement.setCloudProcessingCost,
            "STOP_DOWNLOAD_DATA":queryMeasurement.setDownloadDataCost,
            "STOP_DOWNLOAD_ESTIMATION_RESULT":queryMeasurement.setDownloadCloudEstimationCost,
            "STOP_QUERY_PROCESS":queryMeasurement.setProcessingCost,
            "STOP_CACHE_ANALYSIS":queryMeasurement.setCacheAnalysis,
            "STOP_QUERY_EXECUTION":queryMeasurement.setQueryExecutionCost,
            "STOP_DECISION_PROCESS":queryMeasurement.setDecisionProcessCost,
            "STOP_CACHE_REPLACEMENT":queryMeasurement.setReplacementCost
        }

        timeLapseSetterMap[tag](currentCost)

        return queryMeasurement

    @staticmethod
    def parse(inputFileName):
        exp = Experimentation()

        pendingTimeLapse = {}

        queryMeasurement = QueryMeasurement()

        file = open(inputFileName, 'r')

        lines = file.readlines()

        nbLines = len(lines)

        error = False

        i=0
        while i < nbLines:
            error = False
            lineTag = ExperimentationParser.parseLine(lines[i])

            if lineTag.tag in ExperimentationParser.singleEventTag:
                if lineTag.tag == "CLOUD_PROCESS_RESULT_SIZE":
                    queryMeasurement.setDownloadedResultSize(long(lineTag.value))
                elif lineTag.tag == "CLOUD_COEFFICIENT":
                    queryMeasurement.setCloudCoeff(int(lineTag.value))
                else:
                    if i != 0 and lineTag.tag == "POSED_QUERY":
                        exp.addQueryMeasurement(queryMeasurement)
                        queryMeasurement = QueryMeasurement()
                        queryMeasurement.setQuery(lineTag.extra_arg)

                    if lineTag.tag == "QUERY_CACHE_EXACT_HIT":
                        queryMeasurement.setHitType(HitType.cache_hit)

                    if lineTag.tag == "QUERY_CACHE_EXTENDED_HIT":
                        queryMeasurement.setHitType(HitType.cache_extended_hit)

                    if lineTag.tag == "QUERY_CACHE_PARTIAL_HIT":
                        queryMeasurement.setHitType(HitType.cache_partial_hit)

                    if lineTag.tag == "QUERY_CACHE_MISS":
                        queryMeasurement.setHitType(HitType.cache_miss)

                    if lineTag.tag == "QUERY_PROCESS_ON_CLOUD":
                        queryMeasurement.setProcessedOnCloud(True)

                    ExperimentationParser.execSingleTagFunc(exp,lineTag.tag)

            elif lineTag.tag in ExperimentationParser.timeLapseTags:
                step = lineTag.tag.split('_')[0]
                if step == "START":
                    pendingTimeLapse[lineTag.tag] = long(lineTag.value)
                elif step == "STOP":
                    startingTag = lineTag.tag.replace("STOP","START")
                    timeLapse = long(lineTag.value) - long(pendingTimeLapse[startingTag])
                    pendingTimeLapse.pop(startingTag,-1)
                    currentCost = Cost()
                    currentCost.setTime(timeLapse)
                    costsParsed = False
                    while i+1 < nbLines and costsParsed == False:
                        tmpTag = ExperimentationParser.parseLine(lines[i+1])
                        if tmpTag.tag == "COST_ENERGY":
                            currentCost.setEnergy(float(tmpTag.value))
                            i += 1
                        elif tmpTag.tag == "COST_MONEY":
                            currentCost.setMoney(float(tmpTag.value))
                            i += 1
                        else:
                            costsParsed = True

                    ExperimentationParser.execTimeLapseTagFunc(queryMeasurement,lineTag.tag,currentCost)

                    if i+1 == nbLines: #eof
                        exp.addQueryMeasurement(queryMeasurement)


            else:
                error = True
                print "TAG not handled in "+ inputFileName + ": " + lineTag.tag
                raise NotImplementedError("TAG not handled in "+ inputFileName + ": " + lineTag.tag )

            i+=1

            if error:
                pendingTimeLapse.pop(lineTag.tag.replace("STOP","START"),-1)

        if pendingTimeLapse:
            print inputFileName
            print pendingTimeLapse
            raise ValueError("File corrupted: a tag has been started but has not been stopped")

        file.close()

        return exp



