#!/usr/bin/python
# -*- coding: UTF-8 -*-

__author__ = 'Mikael'


import sys, getopt
from os import listdir,makedirs
from os.path import isfile, join, exists, isdir, dirname
from ExperimentationParser import ExperimentationParser
import re

from ChartGenerator import ChartGenerator
from QuerySetFilesGenerator import QuerySetFilesGenerator


def tryint(s):
    try:
        return int(s)
    except:
        return s

def alphanum_key(s):
    return [ tryint(c) for c in re.split('([0-9]+)', s) ]

def buildExperimentationResult(inputPathsNoCache, inputPathsSemanticCache, inputPathsDecisionalSemanticCache, outputPath):

    fileNoCacheSets = []
    fileSemanticCacheSets = []
    fileDecisionalSemanticCacheSets = []


    for path in inputPathsNoCache:
        fileNoCacheSet = [ join(path,f) for f in listdir(path) if isfile(join(path,f)) ]
        if len(fileNoCacheSet) != 11:
            return -1
        fileNoCacheSet.sort(key=alphanum_key)
        fileNoCacheSets.append(fileNoCacheSet)

    for path in inputPathsSemanticCache:
        fileSemanticCacheSet = [ join(path,f) for f in listdir(path) if isfile(join(path,f)) ]
        if len(fileSemanticCacheSet) != 11:
            return -1
        fileSemanticCacheSet.sort(key=alphanum_key)
        fileSemanticCacheSets.append(fileSemanticCacheSet)

    for path in inputPathsDecisionalSemanticCache:
        fileDecisionalSemanticCacheSet = [ join(path,f) for f in listdir(path) if isfile(join(path,f)) ]
        if len(fileDecisionalSemanticCacheSet) != 11:
            return -1
        fileDecisionalSemanticCacheSet.sort(key=alphanum_key)
        fileDecisionalSemanticCacheSets.append(fileDecisionalSemanticCacheSet)

    chartGenerator = ChartGenerator(outputPath)

    print fileNoCacheSets
    print fileDecisionalSemanticCacheSets
    print fileSemanticCacheSets

    semanticCacheExp = []

    i=0
    for fileSet in fileNoCacheSets:
        for file in fileSet:
            try:
                exp = ExperimentationParser.parse(file)
                chartGenerator.addNoCacheExperimentation(i,exp)
            except ValueError, msg:
                print(msg)
        i += 1

    i=0
    for fileSet in fileSemanticCacheSets:
        for file in fileSet:
            try:
                exp = ExperimentationParser.parse(file)

                if i == 0:
                    semanticCacheExp.append(exp)
                chartGenerator.addSemanticCacheExperimentation(i,exp)
            except ValueError, msg:
                print(msg)
        i += 1

    i=0
    for fileSet in fileDecisionalSemanticCacheSets:
        j=0
        for file in fileSet:
            try:
                exp = ExperimentationParser.parse(file)
                #exp.setDiffCloudProcessingTime(semanticCacheExp[j].queryMeasurements)

                for mes in exp.queryMeasurements:
                    if mes.isProcessedOnCloud():
                        for mesSC in semanticCacheExp[j].queryMeasurements:
                            if mes.getQuery() == mesSC.getQuery():
                                mes.cloudProcessing.money = mesSC.cloudProcessing.money

                print "nb posed queries: " + str(exp.nbPosedQueries)
                print "nb extended hit: " + str(exp.nbQueryCacheExtendedHit)
                print "nb failed queries: " + str(exp.nbFailedQueries)
                print "nb queries run on the cloud: " + str(exp.nbQueriesProcessedOnCloud)
                print "nb queries run on the mobile: " + str(exp.nbQueriesProcessedOnMobile)
                chartGenerator.addDecisionalSemanticCacheExperimentation(i,exp)
            except ValueError, msg:
                print(msg)

            j += 1

        i += 1

    chartGenerator.showTimes("Download Time for Fifty Queries Regarding Cache Partial Hit Percentage", "Cache Partial Hit Percentage (%)", "Processing Time (s)")
    #chartGenerator.showEnergy("Energy Consumption For Fifty Queries Regarding Cache Partial Hit Percentage", "Cache Partial Hit percentage (%)", "Energy Consumption (mAh)")
    #chartGenerator.showMoney("Money Cost For Fifty Queries Regarding Cache Partial Hit Percentage", "Cache Partial Hit percentage (%)", "Money Cost ($)")
    #chartGenerator.showCloudProcessingTimes("Percentage of Queries not Meeting the Money Constraint Regarding Cache Extended Hit Percentage", "Cache Extended Hit Percentage (%)", "Percentage of Queries Not Meeting the Constraint (%)")
    #chartGenerator.showPercentQueriesProcessedCloud("Percentages of Queries Processed on the Cloud Regarding Cache Extended Hit Percentage 3", "Cache Extended Hit Percentage (%)", "Percentage of Processed Queries on the Cloud (%)")
    #chartGenerator.showPercentQueriesProcessedMobile("Percentages of Queries Processed on the Mobile Regarding Cache Partial Hit Percentage", "Cache Partial Hit Percentage (%)", "Percentage of Processed Queries on the Mobile Device (%)")
    #chartGenerator.showMobileProcessingTimes("Mobile Processing Time for Fifty Queries Regarding Cache Partial Hit Percentage", "Cache Partial Hit Percentage (%)", "Processing Time (s)")
    #chartGenerator.showSizeQueriesReprocessedOnCloud("Total Downloaded Size of Queries Reprocessed on Cloud Regarding the Percentage of Partial Hit", "Cache Extended Hit Percentage (%)", "Total Downloaded Size (Bytes)")

def printUsage():
    print 'ExperimentationResultBuilder.py -i <inputpath> '
    print 'The input path needs to contain 3 folders named NO_CACHE, SEMANTIC_CACHE, DECISIONAL_SEMANTIC_CACHE'


def generateExpReport(inputFilesPath):
    outputPath = join(inputFilesPath,"Experimentation_Results")

    if not exists(outputPath):
        makedirs(outputPath)

    print 'Input path is:' + inputFilesPath
    print 'Output path is:' + outputPath

    noCacheDirs = []
    semanticCacheDirs = []
    decisionalSemanticCacheDirs = []

    #get no cache path set
    i=0
    noCacheDir = join(inputFilesPath,'NO_CACHE_'+str(i))
    while isdir(noCacheDir):
        noCacheDirs.append(noCacheDir)
        i+=1
        noCacheDir = join(inputFilesPath,'NO_CACHE_'+str(i))

    #get Semantic caching path set
    i=0
    semanticCacheDir = join(inputFilesPath,'SEMANTIC_CACHE_'+str(i))
    while isdir(semanticCacheDir):
        semanticCacheDirs.append(semanticCacheDir)
        i+=1
        semanticCacheDir = join(inputFilesPath,'SEMANTIC_CACHE_'+str(i))


    j=0
    decisionalSemanticCacheDir = join(inputFilesPath,'DECISIONAL_SEMANTIC_CACHE_'+str(j))
    while isdir(decisionalSemanticCacheDir):
        decisionalSemanticCacheDirs.append(decisionalSemanticCacheDir)
        j+=1
        decisionalSemanticCacheDir = join(inputFilesPath,'DECISIONAL_SEMANTIC_CACHE_'+str(j))

    return buildExperimentationResult(noCacheDirs,semanticCacheDirs,decisionalSemanticCacheDirs,outputPath)

def generateExperimentalQuerySets(inputFilesPath):
    #browse the log file to generate one file per type of cache hit or miss
    outputPath = join(dirname(inputFilesPath),"EXP_QUERY_SETS")

    if not exists(outputPath):
        makedirs(outputPath)

    try:
        exp = ExperimentationParser.parse(inputFilesPath)
        generator = QuerySetFilesGenerator(outputPath, exp)
        generator.generateQuerySetFiles()
    except ValueError, msg:
        print(msg)


def main():
    inputFilesPath=''
    outputPath=''
    retVal = 0


    try:
        opts, args = getopt.getopt(sys.argv[1:], "hi:", ["help","ipath="])
    except getopt.GetoptError:
        printUsage()
        return 2

    for opt, arg in opts:
        if opt == '-h':
            printUsage()
            return 0
        elif opt in ('-i','--ipath'):
            inputFilesPath = arg.decode("ISO-8859-15")
        else:
            print 'invalid option or no option specified'
            printUsage()

    if not inputFilesPath:
        print 'the location of the experimentation files must be specified'
        return 4

    #generate exp query sets
    #generateExperimentalQuerySets(inputFilesPath)

    # generates exp
    retVal = generateExpReport(inputFilesPath)
    return retVal



if __name__ == "__main__":
    sys.exit(main())