__author__ = 'Mikael'

from HitType import HitType

class QueryMeasurement:

    _wantedSpeed = 0.00037 #2MB/s = 0.002 B/ns

    def __init__(self):
        self.query = ''
        self.hitType = HitType
        self.downloadedResultSize = 0
        self.cloudCoeff = 5 # used to adapt the time measurements wrapping the cloud processing time measurements
        self.processing = Cost()
        self.computeCloudEstimation = Cost()
        self.downloadCloudEstimation = Cost()
        self.cacheAnalysis = Cost()
        self.mobileProcessing = Cost()
        self.cloudProcessing = Cost()
        self.diffCloudProcessingTime = 0
        self.downloadData = Cost()
        self.queryExecution = Cost()
        self.decisionProcess = Cost()
        self.replacement = Cost()
        self.processedOnCloud = False
        self.reprocessedOnCloud = False

    def __str__(self):
        str_list = []
        str_list.append("Query:" + self.query)
        str_list.append("Hit type:" + str(self.hitType))
        str_list.append("Downloaded Result Size: " + str(self.downloadedResultSize))
        str_list.append("Overall Processing cost: " + str(self.processing))
        str_list.append("Adapted Processing time: " + str(self.getAdaptedProcessingTime()))
        str_list.append("Compute cloud estimation cost: " + str(self.computeCloudEstimation))
        str_list.append("download cloud estimation cost: " + str(self.downloadCloudEstimation))
        str_list.append("Cache Analysis cost: " + str(self.cacheAnalysis))
        str_list.append("Mobile Processing cost: " + str(self.mobileProcessing))
        str_list.append("Cloud Processing cost: " + str(self.cloudProcessing))
        str_list.append("Download data cost: " + str(self.downloadData))
        str_list.append("Query Execution cost: " + str(self.queryExecution))
        str_list.append("Decision Process cost: " + str(self.decisionProcess))
        str_list.append("Replacement: " + str(self.replacement))

        return ''.join([item + '\n' for item in str_list])

    def setProcessingCost(self, cost):
        self.processing = cost

    def setComputeCloudEstimationCost(self, cost):
        self.computeCloudEstimation = cost

    def setDownloadCloudEstimationCost(self, cost):
        self.downloadCloudEstimation = cost

    def setCacheAnalysis(self, cost):
        self.cacheAnalysis = cost

    def setMobileProcessingCost(self, cost):
        self.mobileProcessing = cost

    def setCloudProcessingCost(self, cost):
        self.cloudProcessing = cost

    def setDiffCloudProcessingTime(self, time):
        self.diffCloudProcessingTime = time

    def setDownloadDataCost(self, cost):
        self.downloadData = cost

    def setDownloadedResultSize(self, size):
        self.downloadedResultSize = size

    def setQueryExecutionCost(self, cost):
        self.queryExecution = cost

    def setCloudCoeff(self, coeff):
        self.cloudCoeff = coeff

    def setDecisionProcessCost(self, cost):
        self.decisionProcess = cost

    def setReplacementCost(self, cost):
        self.replacement = cost

    def setQuery(self, query):
        self.query = query

    def setHitType(self, hitType):
        self.hitType = hitType

    def setProcessedOnCloud(self,isProcessed):
        self.processedOnCloud = isProcessed

    def setReprocessedOnCloud(self, isReprocessed):
        self.reprocessedOnCloud = isReprocessed

    def getQuery(self):
        return self.query

    def getHitType(self):
        return self.hitType

    def getDownloadedResultSize(self):
        return self.downloadedResultSize

    def getAdaptedProcessingTime(self):
        return float(self.getProcessing().time) * (self.getSpeed() / float(QueryMeasurement._wantedSpeed))

    def getAdaptedProcessingEnergy(self):
        return float(self.getProcessing().energy) * (self.getSpeed() / float(QueryMeasurement._wantedSpeed))

    def getSpeed(self):
        if self.downloadedResultSize != 0:
            speed = self.downloadedResultSize/float(self.processing.time)
        else:
            speed = 0
        return speed

    def getCloudCoeff(self):
        return self.cloudCoeff

    def getProcessing(self):
        cost = Cost()
        cost.setCost(self.processing.time - (self.getCloudCoeff()-1)* self.getCloudProcessing().time, self.processing.energy, self.processing.money)

        return cost

    def getComputeCloudEstimation(self):
        return self.computeCloudEstimation

    def getDownloadCloudEstimation(self):
        return self.downloadCloudEstimation

    def getCacheAnalysis(self):
        return self.cacheAnalysis

    def getMobileProcessing(self):
        return self.mobileProcessing

    def getCloudProcessing(self):
        return self.cloudProcessing

    def getDiffCloudProcessingTime(self):
        return self.diffCloudProcessingTime

    def getDownloadData(self):
        return self.downloadData

    def getQueryExecution(self):
        cost = Cost()
        if self.queryExecution != 0:
            cost.setCost(self.queryExecution.time - (self.getCloudCoeff()-1)* self.getCloudProcessing().time, self.queryExecution.energy, self.queryExecution.money)
        return cost

    def getDecisionProcess(self):
        return self.decisionProcess

    def getReplacement(self):
        return self.replacement

    def isProcessedOnCloud(self):
        return self.processedOnCloud

    def isReprocessedOnCloud(self):
        return self.reprocessedOnCloud

class Cost:

    def __init__(self):
        self.time = 0
        self.energy = 0
        self.money = 0

    def __str__(self):
        str_list = []
        str_list.append(str(self.time) + ' ns')
        str_list.append(str(self.energy) + ' mAh')
        str_list.append('$' + str(self.money))

        return ''.join([item + ';' for item in str_list])

    def setTime(self, time):
        self.time = time

    def setEnergy(self, energy):
        self.energy = energy

    def setMoney(self, money):
        self.money = money

    def setCost(self,time,energy,money):
        self.time = time
        self.energy = energy
        self.money = money