﻿using OUDBCommonLibs.Interfaces;
using OUDBCommonLibs.Query;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCache.Cache.SemanticCache
{
    public class RenEtAlSemanticCache<RT> : SemanticCacheBase<RT>
        where RT: new()
    {
        public RenEtAlSemanticCache(int maxTuples, int maxRegions)
            : base(maxTuples, maxRegions)
        {

        }

        public override ISemanticCacheTrimmedQuery<RT, PT> TrimQuery<PT>(IQuery<RT, PT> query)
        {
            IPredicate<RT> pred = query.Predicate;

            IQuery<RT> newQ = new SimpleQuery<RT>(pred);
            
            
            return new SemanticCacheTrimmedQuery<RT, PT>(query, null, query);
        }

        public override void AddToCache<PT>(ISemanticCacheRegion<RT, PT> region)
        {
            this.SemanticRegions.Add(region);

            while (this.MaxCachedTuples > this.NumCachedTuples || this.MaxCacheRegions > this.NumCachedRegions)
            {
                ISemanticCacheRegion<RT> oldRegion = this.SemanticRegions.OrderByDescending(r => r.LastUsed).First();
                this.SemanticRegions.Remove(oldRegion);
            }
        }
    }


    public interface IPredicateAnalyzer<RT>
    {

    }

    public class GuoEtAlPredicateAnalyzer<RT>: IPredicateAnalyzer<RT>
    {

        public GuoEtAlPredicateAnalyzer()
        {

        }
    }

    public class AttributeNode<T> 
        where T:IComparable
    {
        public struct Range{
            public T Bound 
            {
                get
                {
                    return mBound;
                }
                set
                {
                    mBound = value;
                }
            }
            private T mBound;

            public bool IsOpenBound 
            {
                get
                {
                    return mIsOpenBound;
                }
                set
                {
                    mIsOpenBound = value;
                }
            }
            private bool mIsOpenBound;

            public Range(T bound, bool isOpenBound)
            {
                mBound = bound;
                mIsOpenBound = isOpenBound;
            }
        }

        public string Attribute{get; private set;} 
        public Range LowLocalRange{get; set;}
        public Range UpLocalRange { get; set; }
        public Range LowGlobalRange { get; set; }
        public Range UpGlobalRange { get; set; }

        public AttributeNode(string attribute)
        {
            this.Attribute = attribute;
        }

        public bool IsValidInRealDomain()
        {
            return (LowGlobalRange.Bound.CompareTo(UpGlobalRange.Bound) < 0
                    || (LowGlobalRange.Bound.CompareTo(UpGlobalRange.Bound) == 0 && !LowGlobalRange.IsOpenBound && !UpGlobalRange.IsOpenBound));
        }

        public bool IsValidInIntegerDomain()
	    {
		    return (LowGlobalRange.Bound.CompareTo(UpGlobalRange.Bound) <= 0);
	    }

        

        public override int GetHashCode()
        {
		    int result = 1;
		   
		    return result;
        }

    }

}
