﻿using OUDBCommonLibs.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCache.Cache.SemanticCache
{
    public abstract class SemanticCacheBase<RT> : ISemanticCache<RT>
    {
        public SemanticCacheBase(int maxTuples, int maxRegions)
        {
            this.MaxCachedTuples = maxTuples;
            this.MaxCacheRegions = maxRegions;
            this.SemanticRegions = new List<ISemanticCacheRegion<RT>>();
        }
        public int MaxCachedTuples
        {
            get;
            protected set;
        }

        public int MaxCacheRegions
        {
            get;
            protected set;
        }

        public int NumCachedTuples
        {
            get
            {
                return this.SemanticRegions.Sum(scr => scr.UnTypedCachedTuples.Count);
            }
        }

        public int NumCachedRegions
        {
            get
            {
                return this.SemanticRegions.Count;
            }
        }
        protected virtual IList<ISemanticCacheRegion<RT>> SemanticRegions
        {
            get;
            set;
        }

        public abstract ISemanticCacheTrimmedQuery<RT, PT> TrimQuery<PT>(IQuery<RT, PT> query)
            where PT : new();

        public abstract void AddToCache<PT>(ISemanticCacheRegion<RT, PT> region) 
            where PT : new();


        public int RegionsInCache
        {
            get { return this.SemanticRegions.Count; }
        }
    }
}
