﻿using OUDBCommonLibs.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCache.Cache.SemanticCache
{
    public class SemanticCacheTrimmedQuery<RT, PT> : ISemanticCacheTrimmedQuery<RT, PT>
        where PT: new()
    {
        public SemanticCacheTrimmedQuery(IQuery<RT,PT> inputQuery, IQuery<RT,PT> probeQuery, IQuery<RT,PT> remainderQuery) 
        {
            this.InputQuery = inputQuery;
            this.ProbeQuery = probeQuery;
            this.RemainderQuery = remainderQuery;
        }

        public IQuery<RT, PT> InputQuery
        {
            get;
            protected set;
        }

        public IQuery<RT, PT> ProbeQuery
        {
            get;
            protected set;
        }

        public IQuery<RT, PT> RemainderQuery
        {
            get;
            protected set;
        }

        public SemanticQueryType Type
        {
            get
            {
                if (this.ProbeQuery == null && this.RemainderQuery != null)
                {
                    return SemanticQueryType.MISS;
                }
                else if (this.ProbeQuery != null && this.RemainderQuery == null && this.ProbeQuery == this.InputQuery)
                {
                    return SemanticQueryType.HIT;
                }
                else if (this.ProbeQuery != null && this.RemainderQuery != null)
                {
                    return SemanticQueryType.EXTENDED;
                }
                else if (this.ProbeQuery != null && this.RemainderQuery == null)
                {
                    return SemanticQueryType.PARTIAL;
                }else{
                    throw new InvalidOperationException("Unknown Trimmed Query type");
                }
            }
        }

        public IQuery<RT> UnTypedInputQuery
        {
            get { return this.InputQuery; }
        }

        public IQuery<RT> UnTypedProbeQuery
        {
            get { return this.ProbeQuery; }
        }

        public IQuery<RT> UnTypedRemainderQuery
        {
            get { return this.RemainderQuery; }
        }
    }
}
