﻿using OUDBCommonLibs.Generators;
using OUDBCommonLibs.Interfaces;
using OUDBCommonLibs.Variables;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Attributes
{
    [AttributeUsage(AttributeTargets.Class, Inherited = true)]
    public class TupleAttribute : System.Attribute
    {
        public TupleAttribute(string relationName)
        {
            this.RelationName = relationName;
        }

        public string RelationName { get; set; }
    }

    [AttributeUsage(AttributeTargets.Property, Inherited=true)]
    public class TuplePropertyAttribute : System.Attribute
    {

    }

    [AttributeUsage(AttributeTargets.Property, Inherited=true)]
    public class RandomTuplePropertyAttribute : TuplePropertyAttribute
    {
        private Type _schemaType;
        public RandomTuplePropertyAttribute(Type schemaType)
        {
            if (!typeof(RandomAttributeSchemaBase).IsAssignableFrom(schemaType)) throw new InvalidOperationException(string.Format("Invalid schema type ({0}) given", schemaType.FullName));

            this._schemaType = schemaType;
        }

        private RandomAttributeSchemaBase _schema = null;
        public RandomAttributeSchemaBase AttributeSchema
        {
            get
            {
                if (this._schema == null)
                {
                    this._schema = Activator.CreateInstance(this._schemaType) as RandomAttributeSchemaBase;
                }

                return this._schema;
            }
        }
    }

    public class RandomNumbericAttributeSchema : RandomAttributeSchemaBase
    {
        public RandomNumbericAttributeSchema(RandomNumericVariableBase variable)
            :base()
        {
            this.Variable = variable;
        }

        public RandomNumericVariableBase Variable { get; protected set; }

        public override IGenerator GetGenerator()
        {
            return new RandomNumericalGenerator(this.Variable);
        }
    }

    public class RandomLowerAlphaStringAttributeSchema : RandomAttributeSchemaBase
    {
        public RandomLowerAlphaStringAttributeSchema(RandomNumericVariableBase randomAlphabetSelector, RandomNumericVariableBase randomLengthSelector)
            :base()
        {
            this.AlphabetSelector = randomAlphabetSelector;
            this.LengthSelector = randomLengthSelector;
        }

        public RandomLowerAlphaStringAttributeSchema(RandomNumericVariableBase randomLengthSelector)
            : this(new UniformVariable(0, 26), randomLengthSelector)
        {
        }

        public RandomNumericVariableBase AlphabetSelector { get; private set; }

        public RandomNumericVariableBase LengthSelector { get; private set; }

        public override IGenerator GetGenerator()
        {
            return new RandomLowerCaseAlphaStringGenerator(this.AlphabetSelector, this.LengthSelector);
        }
    }

    public abstract class RandomAttributeSchemaBase
    {
        public RandomAttributeSchemaBase() { }

        public abstract IGenerator GetGenerator();
    }
}
