﻿using OUDBCommonLibs.Interfaces;
using OUDBCommonLibs.Variables;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Generators
{
    public class RandomSetGenerator<ST, RT> : GeneratorBase<RT>
    {
        public RandomSetGenerator(IList<ST> possible,
                                    RandomNumericVariableBase randomSelector,
                                    RandomNumericVariableBase randomSizeSelector,
                                    Func<IEnumerable<ST>, RT> combinator)
        {
            this.Possible = possible;
            this.PossibleIndexSelector = randomSelector;
            this.SizeSelector = randomSizeSelector;
            this.Combinator = combinator;
        }

        public IList<ST> Possible { get; private set; }

        public RandomNumericVariableBase PossibleIndexSelector { get; private set; }

        public RandomNumericVariableBase SizeSelector { get; private set; }

        public Func<IEnumerable<ST>, RT> Combinator { get; private set; }

        public override RT GenerateValue()
        {
            int length = (int)this.SizeSelector.Sample();

            IList<ST> selectedValues = new List<ST>();

            for (int ndx = 0; ndx < length; ndx++)
            {
                selectedValues.Add(this.Possible[(int)this.PossibleIndexSelector.Sample()]);
            }

            return this.Combinator(selectedValues);
        }
    }
}
