﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Interfaces
{
    public interface IPredicate
    {
        bool Match(object tuple);

        bool Equals(object predicate);
    }
    public interface IPredicate<T> : IPredicate
    {
        bool Match(T tuple);

        bool Equals(IPredicate<T> predicate);

        IList<ITupleAttributeDesc> GetPredicateAttributes<T>();
    }

    public interface IComparableAttributePredicate<RT, AT> : IPredicate<RT>
        where AT : IComparable<AT>
    {
        Expression<Func<RT, AT>> AttributeExpression { get; }

        ComparisonType Type { get; }

        AT CompareValue { get; }
    }

    public interface IBinaryPredicate<T> : IPredicate<T>
    {
        IPredicate<T> Left { get; }

        IPredicate<T> Right { get; }

        BinaryComparisonType Type { get; }
    }

    public interface IUnaryPredicate<T> : IPredicate<T>
    {
        IPredicate<T> Predicate { get; }
    }

    public enum ComparisonType
    {
        EQUAL,
        LESS_THAN,
        GREATER_THAN,
        LESS_THAN_OR_EQUAL,
        GREATER_THAN_OR_EQUAL
    }

    public enum BinaryComparisonType
    {
        AND,
        OR
    }
}
