﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Interfaces
{
    public interface IRelation<T> : ISizeable
    {
        IList<T> Tuples { get; }
        T Add(T tuple);
        void Remove(T tuple);
        IQueryResult<T,PT> Execute<PT>(IQuery<T, PT> query)
            where PT: new();
    }

    public interface IRelationGenerator<T>
    {
        IRelation<T> Generate(int size);
    }
    public interface IComparableAttributeQueryPredicateGenerator<T>
    {
        IPredicate<T> GenerateAttributePredicate<AT>(Expression<Func<T, AT>> attributeExpression, IVariable<double> percentVariable, bool useLowerPortion = true)
            where AT : IComparable<AT>;
        IPredicate<T> GenerateAttributeConjunctionRangePredicate<AT>(Expression<Func<T, AT>> attributeExpression, IVariable<double> percentVariable, IVariable<double> startPercentVariable)
            where AT : IComparable<AT>;
        IPredicate<T> GenerateAttributeDisjunctionRangePredicate<AT>(Expression<Func<T, AT>> attributeExpression, IVariable<double> lowPercentVariable, IVariable<double> highPercentVariable)
            where AT : IComparable<AT>;
        IPredicate<T> GenerateAttributeConjunctionDisjunctionRangePredicate<AT>(Expression<Func<T, AT>> attributeExpression, IVariable<double> lowPercentVariable, IVariable<double> lowStartPercentVariable, IVariable<double> highPercentVariable, IVariable<double> highStartPercentVariable)
            where AT : IComparable<AT>;
    }
}
