﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Interfaces
{
    public interface ISemanticCache<RT>
    {
        ISemanticCacheTrimmedQuery<RT, PT> TrimQuery<PT>(IQuery<RT, PT> query)
            where PT: new();

        void AddToCache<PT>(ISemanticCacheRegion<RT, PT> region)
            where PT : new();

        int RegionsInCache { get; }
    }

    public interface ISemanticCacheRegion<RT>
    {
        IQuery<RT> UnTypedQuery { get; }

        Type ProjectedType { get; }

        IList<object> UnTypedCachedTuples { get; }

        DateTime LastUsed { get; set; }
    }

    public interface ISemanticCacheRegion<RT, PT> : ISemanticCacheRegion<RT>
        where PT : new()
    {
        IQuery<RT, PT> Query { get; }

        IList<PT> CachedTuples { get; }
    }

    public interface ISemanticCacheTrimmedQuery<RT>
    {
        IQuery<RT> UnTypedInputQuery { get; }

        IQuery<RT> UnTypedProbeQuery { get; }

        IQuery<RT> UnTypedRemainderQuery { get; }

        SemanticQueryType Type { get; }
    }


    public interface ISemanticCacheTrimmedQuery<RT, PT> : ISemanticCacheTrimmedQuery<RT>
        where PT : new()
    {
        IQuery<RT, PT> InputQuery { get; }

        IQuery<RT, PT> ProbeQuery { get; }

        IQuery<RT, PT> RemainderQuery { get; }
    }

    public enum SemanticQueryType
    {
        HIT,
        MISS,
        PARTIAL,
        EXTENDED
    }
}
