﻿using OUDBCommonLibs.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Predicates
{
    public class BinaryPredicate<T> : IBinaryPredicate<T>
    {
        public BinaryPredicate(IPredicate<T> left, IPredicate<T> right, BinaryComparisonType type)
        {
            this.Left = left;
            this.Right = right;
            this.Type = type;

        }
        public IPredicate<T> Left
        {
            get;
            private set;
        }

        public IPredicate<T> Right
        {
            get;
            private set;
        }

        public BinaryComparisonType Type
        {
            get;
            private set;
        }

        public bool Match(T tuple)
        {
            switch (this.Type)
            {
                case BinaryComparisonType.OR:
                    return this.Left.Match(tuple) || this.Right.Match(tuple);

                case BinaryComparisonType.AND:
                default:
                    return this.Left.Match(tuple) && this.Right.Match(tuple);
            }
        }

        public override string ToString()
        {
            string compareOp;
            switch (this.Type)
            {
                case BinaryComparisonType.OR:
                    compareOp = "OR";
                    break;

                case BinaryComparisonType.AND:
                default:
                    compareOp= "AND";
                    break;
            }

            return string.Format("({0}) {1} ({2})", this.Left.ToString(), compareOp, this.Right.ToString());
        }

        public bool Match(object tuple)
        {
            if (tuple is T)
            {
                return this.Match((T)tuple);
            }
            else
            {
                return false;
            }
        }

        public override bool Equals(object obj)
        {
            // If parameter is null return false.
            if (obj == null)
            {
                return false;
            }

            // If parameter cannot be cast return false.
            BinaryPredicate<T> otherPred = obj as BinaryPredicate<T>;
            if ((System.Object)otherPred == null)
            {
                return false;
            }

            // Return true if the fields match:
            return this.Equals(otherPred);
        }

        public bool Equals(IPredicate<T> iOtherPred)
        {
            IBinaryPredicate<T> otherPred = iOtherPred as IBinaryPredicate<T>;
            // If parameter is null return false:
            if ((object)otherPred == null)
            {
                return false;
            }

            // Return true if the fields match:
            if (this.Type != otherPred.Type) return false;
            if (!this.Left.Equals(otherPred.Left)) return false;
            if (!this.Right.Equals(otherPred.Right)) return false;

            return true;
        }

        public override int GetHashCode()
        {
            var result = 0;
            result = (result * 397) ^ this.Type.GetHashCode();
            result = (result * 397) ^ this.Left.GetHashCode();
            result = (result * 397) ^ this.Right.GetHashCode();

            return result;
        }

        public IList<ITupleAttributeDesc> GetPredicateAttributes<RT>()
        {
            List<ITupleAttributeDesc> attributeDescs = new List<ITupleAttributeDesc>();

            attributeDescs.AddRange(this.Left.GetPredicateAttributes<RT>());
            attributeDescs.AddRange(this.Right.GetPredicateAttributes<RT>());
           

            return attributeDescs;
        }
    }
}
