﻿using OUDBCommonLibs.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Predicates
{
    public class NegatePredicate<T> : IUnaryPredicate<T>
    {
        public NegatePredicate(IPredicate<T> predicate)
        {
            this.Predicate = predicate;
        }
        public IPredicate<T> Predicate
        {
            get;
            private set;
        }

        public bool Match(T tuple)
        {
            return !this.Predicate.Match(tuple);
        }

        public override string ToString()
        {
            return string.Format("NOT ({0})", this.Predicate.ToString());
        }

        public bool Match(object tuple)
        {
            if (tuple is T)
            {
                return this.Match((T)tuple);
            }
            else
            {
                return false;
            }
        }

        public override bool Equals(object obj)
        {
            // If parameter is null return false.
            if (obj == null)
            {
                return false;
            }

            // If parameter cannot be cast return false.
            NegatePredicate<T> otherPred = obj as NegatePredicate<T>;
            if ((System.Object)otherPred == null)
            {
                return false;
            }

            // Return true if the fields match:
            return this.Equals(otherPred);
        }

        public bool Equals(IPredicate<T> obj)
        {
            NegatePredicate<T> otherPred = obj as NegatePredicate<T>;
            // If parameter is null return false:
            if ((object)otherPred == null)
            {
                return false;
            }

            // Return true if the fields match:
            if (this.Predicate != otherPred.Predicate) return false;

            return true;
        }

        public override int GetHashCode()
        {
            var result = 0;
            result = (result * 397) ^ this.Predicate.GetHashCode();

            return result;
        }

        public IList<ITupleAttributeDesc> GetPredicateAttributes<RT>()
        {
            List<ITupleAttributeDesc> attributeDescs = new List<ITupleAttributeDesc>();

            attributeDescs.AddRange(this.Predicate.GetPredicateAttributes<RT>());
            
            return attributeDescs;
        }
    }
}
