﻿using OUDBCommonLibs.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;
using OUDBCommonLibs.ExtensionMethods;
using System.Reflection;
using OUDBCommonLibs.Attributes;
using OUDBCommonLibs.Tuple;

namespace OUDBCommonLibs.Query
{
    public class ProjectionQuery<RT, PT> : IQuery<RT, PT>
        where PT : new()
    {
        public ProjectionQuery() { }

        public ProjectionQuery(IPredicate<RT> predicate, IProjection<RT,PT> projection)
            :this()
        {
            this.Predicate = predicate;
            this.Projection = projection;
        }

        public IPredicate<RT> Predicate
        {
            get;
            protected set;
        }

        public IProjection<RT, PT> Projection 
        { 
            get; 
            protected set; 
        }

        public override string ToString()
        {
            string propertiesString = this.Projection.ToString();
            string relationName = typeof(RT).ResolveRelationName();
            string predString = this.Predicate.ToString();
            return string.Format("SELECT {0} FROM {1} WHERE {2}", propertiesString, relationName, predString);
        }


        public IPredicate UnTypedPredicate
        {
            get { return this.Predicate; }
        }
    }

    public class SimpleQuery<RT> : ProjectionQuery<RT, RT>
        where RT : new()
    {
        public SimpleQuery() : base() { }

        public SimpleQuery(IPredicate<RT> predicate)
            :this()
        {
            this.Predicate = predicate;
            this.Projection = new Projection<RT, RT>();

            Type type = typeof(RT);
            foreach (PropertyInfo pInfo in type.GetProperties())
            {
                Attribute attr = pInfo.GetCustomAttribute(typeof(TuplePropertyAttribute));

                if (attr != null)
                {
                    MethodInfo getMethod = pInfo.GetGetMethod();
                    MethodInfo setMethod = pInfo.GetSetMethod();
                    Action<RT, RT> simpleProjection = (RT left, RT right) => setMethod.Invoke(right, new object[] { getMethod.Invoke(left, null) });

                    ITupleAttributeDesc attributeDesc = new TupleAttributeDesc(pInfo.Name, pInfo.ReflectedType, pInfo.PropertyType);
                    this.Projection.AttributeProjections.Add(new ProjectionDesc<RT, RT>(attributeDesc, attributeDesc, simpleProjection));
                }
            }
        }
    }
}
