﻿using OUDBCommonLibs.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using OUDBCommonLibs.ExtensionMethods;
using Newtonsoft.Json;
using OUDBCommonLibs.Query;

namespace OUDBCommonLibs.Relation
{
    public class Relation<T> : IRelation<T>
    {
        public Relation()
        {
            this.Tuples = new List<T>();
        }

        public Relation(params T[] tuples)
            :this()
        {
            foreach(T t in tuples){
                this.Add(t);
            }
        }

        public IList<T> Tuples
        {
            get;
            private set;
        }

        public T Add(T tuple)
        {
            this.Tuples.Add(tuple);

            return tuple;

        }

        public void Remove(T tuple)
        {
            this.Tuples.Remove(tuple);
        }

        public IQueryResult<T, PT> Execute<PT>(IQuery<T,PT> query)
            where PT : new()
        {
            IProjection<T,PT> projection = query.Projection;
            IQueryResult<T, PT> result = new QueryResult<T, PT>(query);

            foreach (T tuple in this.Tuples)
            {
                if (query.Predicate.Match(tuple))
                {
                    result.Tuples.Add(projection.Project(tuple));
                }
            }

            return result;
        }

        public int GetSizeInBytes()
        {
            return this.Tuples.Sum(t =>
            {
                if (t is ISizeable)
                {
                    return (t as ISizeable).GetSizeInBytes();
                }
                else
                {
                    string json = JsonConvert.SerializeObject(t);
                    byte[] bytes = UTF8Encoding.UTF8.GetBytes(json);

                    return bytes.Length;
                }
                
            });
        }

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine(this.ResolveRelationName());

            foreach (T tuple in this.Tuples)
            {
                sb.Append("\t");
                sb.AppendLine(tuple.TupleToString());
            }

            return sb.ToString();
        }
    }
}
