﻿using OUDBCommonLibs.Attributes;
using OUDBCommonLibs.Interfaces;
using OUDBCommonLibs.Predicates;
using OUDBCommonLibs.Variables;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;
using OUDBCommonLibs.ExtensionMethods;
using System.Reflection;

namespace OUDBCommonLibs.Relation
{
    public class RelationGenerator<RT> : IRelationGenerator<RT>
        where RT : new()
    {
        public RelationGenerator()
        {
        }

        public IRelation<RT> Generate(int size)
        {
            Relation<RT> relation = new Relation<RT>();

            Type relationType = typeof(RT);
            List<Action<RT>> setGenProperies = new List<Action<RT>>();

            foreach (PropertyInfo pInfo in relationType.GetProperties())
            {
                TuplePropertyAttribute tuplePropertyAttribute = pInfo.GetCustomAttribute(typeof(TuplePropertyAttribute)) as TuplePropertyAttribute;

                MethodInfo setMethod = pInfo.GetSetMethod();
                Type parameterType = pInfo.PropertyType;

                if (tuplePropertyAttribute is RandomTuplePropertyAttribute)
                {
                    RandomTuplePropertyAttribute randomTuplePropertyAttribute = tuplePropertyAttribute as RandomTuplePropertyAttribute;
                    IGenerator generator = randomTuplePropertyAttribute.AttributeSchema.GetGenerator();
                    
                    Action<RT> setProperty = (RT t) =>
                    {
                        object value = generator.GenerateObjectValue();
                        setMethod.Invoke(t, new object[] { Convert.ChangeType(value, parameterType) });
                    };

                    setGenProperies.Add(setProperty);
                }
                else if(tuplePropertyAttribute != null)
                {
                    //Not random, use default value
                    Action<RT> setProperty = (RT t) =>
                    {
                    
                        object value = null;
                        if (t.GetType().IsValueType)
                        {
                            value = Activator.CreateInstance(t.GetType());
                        }

                        setMethod.Invoke(t, new object[] { value });
                    };

                    setGenProperies.Add(setProperty);
                }
            }

            Func<RT> tupleGenFunc = () =>
            {
                RT value = new RT();
                foreach (Action<RT> setAction in setGenProperies)
                {
                    setAction(value);
                }

                return value;
            };


            for (int ndx = 0; ndx < size; ndx++)
            {

                RT tuple = tupleGenFunc();
                relation.Add(tuple);
            }

            return relation;
        }
        
    }
}
