﻿using OUDBCommonLibs.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;
using OUDBCommonLibs.ExtensionMethods;
using System.Reflection;

namespace OUDBCommonLibs.Tuple
{
    public class Projection<RT, PT> : IProjection<RT, PT>
        where PT : new()
    {
        public Projection()
        {
            this.AttributeProjections = new List<IProjectionDesc<RT, PT>>();
        }

        public PT Project(RT tuple)
        {
            PT projectedTuple = new PT();

            foreach (IProjectionDesc<RT, PT> projection in this.AttributeProjections)
            {
                projection.ProjectAttributeFunc(tuple, projectedTuple);
            }

            return projectedTuple; 
        }

        public  IList<IProjectionDesc<RT, PT>> AttributeProjections{ get; private set;}

        public void AddProjectedAttribute<RAT, AT>(Expression<Func<RT, RAT>> relationAttributeExpression, Expression<Func<PT, AT>> projectedAttributeExpression, Expression<Func<RAT, AT>> projectionExpression)
        {
            ITupleAttributeDesc relationAttribute = relationAttributeExpression.ResolveAttributeDesc();
            ITupleAttributeDesc projectionAttribute = projectedAttributeExpression.ResolveAttributeDesc();

            Func<RT, RAT> getRelationAttr = relationAttributeExpression.Compile();
            Func<RAT, AT> projection = projectionExpression.Compile();

            PropertyInfo pInfo = projectionAttribute.RelationType.GetProperty(projectionAttribute.Name, projectionAttribute.AttributeType);

            Action<RT, PT> projectionAttributeAction = (r, t) => pInfo.GetSetMethod().Invoke(t, new object[]{projection(getRelationAttr(r))});

            this.AttributeProjections.Add(new ProjectionDesc<RT, PT>(relationAttribute, projectionAttribute, projectionAttributeAction));
        }

        public override string ToString()
        {
            return string.Join(", ", this.AttributeProjections.Select(x => x.ToString()));
        }

    }

    public class ProjectionDesc<RT, PT> : IProjectionDesc<RT, PT>
        where PT : new()
    {
        public ProjectionDesc(ITupleAttributeDesc relationAttrDesc, ITupleAttributeDesc projectionAttrDesc, Action<RT, PT> projectAttrFunc)
        {
            this.RelationAttribute = relationAttrDesc;
            this.ProjectionAttribute = projectionAttrDesc;
            this.ProjectAttributeFunc = projectAttrFunc;
        }

        public ITupleAttributeDesc RelationAttribute
        {
            get;
            private set;
        }

        public ITupleAttributeDesc ProjectionAttribute
        {
            get;
            private set;
        }

        public Action<RT, PT> ProjectAttributeFunc { get; private set; }

        public override string ToString()
        {
            return this.ProjectionAttribute.Name;
        }
    }
}
