﻿using MathNet.Numerics.Distributions;
using OUDBCommonLibs.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Variables
{
    public class ExponentialVariable : RandomNumericVariableWithMeanBase, IQuantile<double>
    {
        private Exponential _variable;
        public ExponentialVariable(double mean = 0, double bound = double.PositiveInfinity)
            :base()
        {
            this.Mean = mean;
            this.Bound = bound;

            this._variable = new Exponential(this.Rate);
        }

        public override double Sample()
        {
            while (true)
            {
                double value = this._variable.Sample();
                if (value <= this.Bound) return value;
            }
        }

        public override double Mean { get; protected set; }

        public double Bound { get; private set; }

        public double Rate { get { return 1.0 / this.Mean; } }

        public double CalculateValue(double probability)
        {
            if (probability < 0.0 || probability > 1.0) throw new InvalidOperationException(string.Format("Invalid probability value ({0}), must be between 0 and 1", probability));

            return this._variable.InverseCumulativeDistribution(probability);
        }


        public double CalculateProbability(double value)
        {
            return this._variable.CumulativeDistribution(value);
        }
    }
}
