﻿using MathNet.Numerics.Distributions;
using OUDBCommonLibs.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Variables
{
    public class LogNormalVariable : RandomNumericVariableWithMeanBase, IQuantile<double>
    {
        private LogNormal _variable;
        public LogNormalVariable(double mean = 0, double variance = 1, double bound = double.PositiveInfinity)
            :base()
        {
            this.Mean = mean;
            this.Variance = variance;
            this.Bound = bound;

            this._variable = new LogNormal(this.LogScale, this.Shape);
        }

        public override double Sample()
        {
            while (true)
            {
                double value = this._variable.Sample();
                if (value <= this.Bound) return value;
            }
        }

        public override double Mean { get; protected set; }

        public double Variance { get; private set; }

        public double Bound { get; private set; }

        public double LogScale { get { return Math.Log(this.Mean / (Math.Sqrt(1 + (this.Variance / Math.Pow(this.Mean, 2.0))))); } }

        public double Shape { get { return Math.Sqrt(Math.Log(1 + (this.Variance / Math.Pow(this.Mean, 2.0)))); } }

        public double CalculateValue(double probability)
        {
            if (probability < 0.0 || probability > 1.0) throw new InvalidOperationException(string.Format("Invalid probability value ({0}), must be between 0 and 1", probability));

            return this._variable.InverseCumulativeDistribution(probability);
        }

        public double CalculateProbability(double value)
        {
            return this._variable.CumulativeDistribution(value);
        }
    }
}
