﻿using MathNet.Numerics.Distributions;
using OUDBCommonLibs.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Variables
{
    public class NormalVariable : RandomNumericVariableWithMeanBase, IQuantile<double>
    {
        private Normal _variable;

        public NormalVariable(double mean = 0, double variance = 1, double bound = double.PositiveInfinity)
            :base()
        {
            this.Mean = mean;
            this.Variance = variance;
            this.Bound = bound;

            this._variable = new Normal(this.Mean, this.StdDev);
        }

        public override double Sample()
        {
            while (true)
            {
                double value = this._variable.Sample();

                if (Math.Abs(value - this.Mean) <= this.Bound)
                {
                    return value;
                }
            }
        }

        public override double Mean { get; protected set; }

        public double Variance { get; private set; }

        public double StdDev { get { return Math.Sqrt(this.Variance); } }

        public double Bound { get; private set; }

        public double CalculateValue(double probability)
        {
            if (probability < 0.0 || probability > 1.0) throw new InvalidOperationException(string.Format("Invalid probability value ({0}), must be between 0 and 1", probability));

            return this._variable.InverseCumulativeDistribution(probability);
        }

        public double CalculateProbability(double value)
        {
            return this._variable.CumulativeDistribution(value);
        }
    }
}
