﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Variables
{
    public class SequentialVariable : RandomNumericVariableBase
    {
        private int _currentConsequtiveCount;
        private double _currentValue;

        public SequentialVariable(double firstValue, double lastValueExclusive, double increment = 1, int consecutiveCount = 1)
            :base()
        {
            if (firstValue > lastValueExclusive) throw new InvalidOperationException(string.Format("Invalid First({0}) and Last({1}) combination for a SequentialVariable", firstValue, lastValueExclusive));

            this.FirstValue = firstValue;
            this.LastValueExclusive = lastValueExclusive;
            this.IncrementVariable = new ConstantVariable(increment);
            this.ConsecutiveCount = consecutiveCount;

            this._currentConsequtiveCount = 0;
            this._currentValue = firstValue;
        }

        public override double Sample()
        {
            double value = this._currentValue;

            if (++this._currentConsequtiveCount == this.ConsecutiveCount)
            {
                this._currentConsequtiveCount = 0;
                this._currentValue += this.IncrementVariable.Sample();

                if (this._currentValue >= this.LastValueExclusive)
                {
                    this._currentValue = this.FirstValue + (this._currentValue - this.LastValueExclusive);
                }
            }

            return value;
        }

        public double FirstValue { get; private set; }

        public double LastValueExclusive { get; private set; }

        public RandomNumericVariableBase IncrementVariable { get; private set; }

        public int ConsecutiveCount { get; private set; }
    }
}
