﻿using MathNet.Numerics.Distributions;
using OUDBCommonLibs.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Variables
{
    /// See http://en.wikipedia.org/wiki/Triangular_distribution for a description of the triangular probability distribution and the algorithm for generating one.
    public class TriangularVariable : RandomNumericVariableWithMeanBase, IQuantile<double>
    {
        private Triangular _variable;
        public TriangularVariable(double min, double max, double mode)
        {
            if (min > max) throw new InvalidOperationException(string.Format("Invalid Min({0}) and Max({1}) combination for a TriangularVariable", min, max));

            if (mode < min || mode > max) throw new InvalidOperationException(string.Format("Invalid Mode{2} for Min({0}) and Max({1}) combination for a TriangularVariable", min, max, mode));

            this.Min = min;
            this.Max = max;
            this.Mode = mode;
            
            this._variable = new Triangular(this.Min, this.Max, this.Mode);
        }
        public override double Sample()
        {
            return this._variable.Sample();
        }

        public double Min { get; private set; }

        public double Max { get; private set; }

        public double Mode { get; private set; }

        public double CalculateValue(double probability)
        {
            if (probability < 0.0 || probability > 1.0) throw new InvalidOperationException(string.Format("Invalid probability value ({0}), must be between 0 and 1", probability));

            return this._variable.InverseCumulativeDistribution(probability);
        }

        public double CalculateProbability(double value)
        {
            return this._variable.CumulativeDistribution(value);
        }

        public override double Mean
        {
            get { return this._variable.Mean; }
            protected set { throw new NotImplementedException(); }
        }
    }
}
