﻿using MathNet.Numerics.Distributions;
using OUDBCommonLibs.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Variables
{
    public class UniformVariable : RandomNumericVariableWithMeanBase, IQuantile<double>
    {
        private ContinuousUniform _variable;

        public UniformVariable(double min = 0, double max = 1)
            :base()
        {
            if (min > max) throw new InvalidOperationException(string.Format("Invalid Min({0}) and Max({1}) combination for a UniformRandomVaribale", min, max));

            this.Min = min;
            this.Max = max;
            
            this._variable = new ContinuousUniform(this.Min, this.Max);
        }

        public override double Sample()
        {
            return this._variable.Sample();
        }

        public double Min { get; private set; }

        public double Max { get; private set; }

        public double CalculateValue(double probability)
        {
            if (probability < 0.0 || probability > 1.0) throw new InvalidOperationException(string.Format("Invalid probability value ({0}), must be between 0 and 1", probability));
            
            return this._variable.InverseCumulativeDistribution(probability);
        }

        public double CalculateProbability(double value)
        {
            return this._variable.CumulativeDistribution(value);
        }

        public override double Mean
        {
            get { return this._variable.Mean; }
            protected set { throw new NotImplementedException(); }
        }
    }
}
