﻿using MathNet.Numerics;
using MathNet.Numerics.Distributions;
using OUDBCommonLibs.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBCommonLibs.Variables
{
    public class WeibullVariable : RandomNumericVariableWithMeanBase, IQuantile<double>
    {
        private Weibull _variable;

        public WeibullVariable(double mean = 1.0, double shape = 1.0, double bound = double.PositiveInfinity)
            :base()
        {
            this.Mean = mean;
            this.Shape = shape;
            this.Bound = bound;

            this._variable = new Weibull(this.Shape, this.Scale);
        }

        public override double Sample()
        {
            while (true)
            {
                double value = this._variable.Sample();
                if (value <= this.Bound) return value;
            }
        }

        public override double Mean { get; protected set; }

        public double Shape { get; private set; }

        public double Bound { get; private set; }

        public double Scale { get { return this.Mean / SpecialFunctions.Gamma(1.0 + (1.0 / this.Shape)); } }

        public double CalculateValue(double probability)
        {
            if (probability < 0.0 || probability > 1.0) throw new InvalidOperationException(string.Format("Invalid probability value ({0}), must be between 0 and 1", probability));
            
            double exponent = 1.0 / this.Shape;

            return Math.Pow(-1.0 * Math.Log(1.0 - probability), exponent);
        }

        public double CalculateProbability(double value)
        {
            return this._variable.CumulativeDistribution(value);
        }
    }
}
