﻿
using OUDBCommonLibs.Interfaces;
using OUDBCommonLibs.Variables;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OUDBExperiments.Implementaions
{
    /// <summary>
    /// 
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class CachedQueryBatchExperiment<T> : ICachedQueryBatchExperiment<T>
    {
        public CachedQueryBatchExperiment(int maxCacheRegions, int relationSize, int maxCachedTuples, 
                                            RandomNumericVariableWithMeanBase queryPercentOfTotalRelation, double experimentalPercent, 
                                            SemanticQueryType experimentalVariable, double sematincHitPercent)
        {
            if (experimentalVariable == SemanticQueryType.MISS) throw new InvalidOperationException("Does not support experimenting on Cache Miss Percent");
            this.MaxCacheRegions = maxCacheRegions;
            this.RelationSize = relationSize;
            this.MaxCachedTuples = maxCachedTuples;
            this.PercentExperimental = experimentalPercent;
            this.ExperimentalVariable = experimentalVariable;
            this.PercentSemanticHit = sematincHitPercent;
            this.QueryPercentOfTotalRelation = queryPercentOfTotalRelation;
        }

        public int MaxCacheRegions { get; set; }
        public int RelationSize { get; set; }
        public int MaxCachedTuples { get; set; }
        public double PercentExperimental { get; set; }
        public double PercentMiss
        {
            get
            {
                return 1.0 - this.PercentExperimental;
            }
        }
        public SemanticQueryType ExperimentalVariable { get; set; }
        public double PercentSemanticHit { get; set; }
        public RandomNumericVariableWithMeanBase QueryPercentOfTotalRelation
        {
            get;
            set;
        }
        public int MeanTuplesPerQuery { get { return (int)Math.Floor((double)this.RelationSize * this.QueryPercentOfTotalRelation.Mean); } }
        public double CacheablePercent { get { return (double)this.MaxCachedTuples / (double)this.RelationSize; } }
        public double AverageQueryPercentOfCache { get { return (double)this.MeanTuplesPerQuery / this.MaxCachedTuples; } }
        public int MaxCacheableRegions { get { return (int)Math.Floor(1.0 / this.AverageQueryPercentOfCache); } }
        public int MaxNumUniqueRegions { get { return (int)Math.Floor(1.0 / this.QueryPercentOfTotalRelation.Mean); } }
    }
}
