﻿using OUDBCommonLibs.Attributes;
using OUDBCommonLibs.Interfaces;
using OUDBCommonLibs.Predicates;
using OUDBCommonLibs.Variables;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;
using OUDBCommonLibs.ExtensionMethods;
using OUDBCommonLibs.Tuple;
using OUDBCommonLibs.Relation;
using OUDBCommonLibs.Generators;
using OUDBCommonLibs.Util;
using OUDBCommonLibs.IntervalTree;
using OUDBQueryGeneration.QueryBatchGenerator;
using OUDBCommonLibs.Query;
using OUDBExperiments.Implementaions;

namespace OUDBQueryGeneration
{
    class Program
    {
        public static void Main(string[] args)
        {
            IPredicate<test> a = new ComparableAttributePredicate<test,double>(t => t.Weight, ComparisonType.GREATER_THAN, 130.0);
            IPredicate<test> b = new ComparableAttributePredicate<test, double>(t => t.Weight, ComparisonType.LESS_THAN, 180.0);
            IPredicate<test> p1 = new BinaryPredicate<test>(a,b,BinaryComparisonType.AND);
            IPredicate<test> c = new ComparableAttributePredicate<test, double>(t => t.Weight, ComparisonType.GREATER_THAN, 150.0);
            IPredicate<test> p2 = new BinaryPredicate<test>(c, p1, BinaryComparisonType.AND);
            //IPredicate<test> nP = new NegatePredicate<test>(p);
            double v = p2.PercentOfRelation();

            IList<Interval<double>> intervals = p2.ComputeIntervals<test,double>();
            IPredicate<test> pred = intervals.ToPredicate<test, double>(t => t.Weight);

            ICachedQueryBatchExperiment<test> exp = new CachedQueryBatchExperiment<test>(10, 100000, 50000, new UniformVariable(0.025, 0.075), 0.4, SemanticQueryType.PARTIAL, 0.3);

            IQueryBatchGenerator<test> gen = new SingleAttributeCachedQueryBatchGenerator<test, double>(exp, t => t.Weight);

            var lst = gen.GenerateQueryBatch(100);

            ComparableAttributePredicateGenerator<test> predGen = new ComparableAttributePredicateGenerator<test>();
            IPredicate<test> predicate = predGen.GenerateAttributePredicate(t => t.Weight, new ConstantVariable(0.1), false);

            Console.WriteLine(predicate.ToString());


            IList<Interval<double>> list = predicate.ComputeIntervals<test, double>();

            Console.WriteLine(string.Join(", ", list.Select(t => string.Format("[{0},{1}]", t.Start, t.End))));
            //Projection<test, proj_test> proj = new Projection<test, proj_test>();
            //proj.AddProjectedAttribute(t => t.Id, pt => pt.Id, id => id);

            //Console.WriteLine(proj.ToString());
            //Relation<test> relation = new Relation<test>();

            //string name = relation.ResolveRelationName();
            //Expression<Func<test, int>> exp = (test t) => t.Age;

            //Console.WriteLine(exp.ResolveAttributeName());

            //IAttributeDesc desc = exp.GetTupleAttribute();
            //TestRelationGeneration();
            //for (int i = 0; i < 10; i++) { TestNormalDistribution(); }
            //for (int i = 0; i < 10; i++) { TestNormalQuantile(); }
            //for (int i = 0; i < 10; i++) { TestNormalQuantileRange(); }
            //for (int i = 0; i < 10; i++) { TestUniformQuantile(); }
            //for (int i = 0; i < 10; i++) { TestTriangleQuantile(); }
            //for (int i = 0; i < 10; i++) { TestTriangleQuantileRange(); }
            //for (int i = 0; i < 10; i++) { TestExpQuantile(); }
            //for (int i = 0; i < 10; i++) { TestParetoQuantile(); }
            //for (int i = 0; i < 10; i++) { TestWeibullQuantile(); }
            //TestQueryToString();
            //TestQueryExecute();
            //for (int i = 0; i < 10; i++) { TestQueryGenerateExecute(); }

           // TestQueryGenerateExecute();
            //TestTypedRelationGeneration();
            Console.ReadLine();
        }

        public static void TestQueryGenerateExecute()
        {
            //GeneratedRelationDesc r = new GeneratedRelationDesc("test",
            //                            new RandomNumericalGeneratedAttributeDesc("Id", typeof(int), new SequentialVariable(0, double.MaxValue)),
            //                            new RandomLowerCaseAlphaGeneratedAttributeDesc("Name", new NormalVariable(8, 3)),
            //                            new RandomNumericalGeneratedAttributeDesc("Age", typeof(int), new NormalVariable(30, 30)),
            //                            new RandomNumericalGeneratedAttributeDesc("Weight", typeof(double), new UniformVariable(100, 200)),
            //                            new RandomNumericalGeneratedAttributeDesc("Height", typeof(float), new TriangularVariable(60, 84, 68)));

           // GeneratedRelationDesc r = new TypedGeneratedRelationDesc<test>("test");

            //IPredicate predicate = new AttributePredicate(new AttributeDesc("Age", typeof(int)), ComparisonType.GREATER_THAN_OR_EQUAL, Comparer<object>.Create((left, right) => ((int)left).CompareTo((int)right)), () => 7);
            // IPredicate left = new IntegerAttributePredicate(new AttributeDesc("Age", typeof(int)), ComparisonType.GREATER_THAN_OR_EQUAL, 9);
            //IPredicate right = new IntegerAttributePredicate(new AttributeDesc("Weight", typeof(double)), ComparisonType.LESS_THAN, 175);
            //IPredicate predicate = new BooleanPredicate(left, right, BooleanComparisonType.AND);

            //IPredicate predicate = new DoubleAttributePredicate(new AttributeDesc("Weight", typeof(double)), ComparisonType.LESS_THAN, 175);
            //IPredicate predicate = new RandomDoubleAttributePredicate(new AttributeDesc("Weight", typeof(double)), ComparisonType.LESS_THAN, new UniformRandom(150, 175));
            //QueryDesc query = new QueryDesc(r, predicate);

            //TypedRelationGenerator<test> relationGen = new TypedRelationGenerator<test>();

            int total = 100000;
            //IGeneratedRelation genRelation = relationGen.Generate(total);
            //IQueryDesc query = new QueryDesc(genRelation, relationGen.GenerateAttributeConjunctionRangePredicate(genRelation.GenerateAttributes.First(a => a.Name == "Weight"), new ConstantVariable(0.4), new ConstantVariable(0.2)));
            //IQueryDesc query = new QueryDesc(genRelation, relationGen.GenerateAttributePredicate(genRelation.GenerateAttributes.First(a => a.Name == "Weight"), new NormalVariable(0.6, 0.01), true));
            //IQueryDesc query = new QueryDesc(genRelation, relationGen.GenerateAttributeDisjunctionRangePredicate(genRelation.GenerateAttributes.First(a => a.Name == "Weight"), new UniformRandom(0.4, 0.5), new ConstantVariable(0.4)));
            //IQueryDesc query = new QueryDesc(genRelation, relationGen.GenerateAttributeConjunctionDisjunctionRangePredicate(t => t.Weight, new ConstantVariable(0.1),new ConstantVariable(0.3), new ConstantVariable(0.4), new ConstantVariable(0.5)));
            //ITypedQueryDesc<test> query = new TypedQueryDesc<test>(relationGen.GenerateAttributeConjunctionDisjunctionRangePredicate(t => t.Weight, new ConstantVariable(0.1), new ConstantVariable(0.3), new ConstantVariable(0.4), new ConstantVariable(0.5)));

            //IPredicate left = relationGen.GenerateAttributeConjunctionDisjunctionRangePredicate(genRelation.GenerateAttributes.First(a => a.Name == "Height"), new ConstantVariable(0.1),new ConstantVariable(0.3), new ConstantVariable(0.4), new ConstantVariable(0.5));
            //IPredicate right = relationGen.GenerateAttributeConjunctionRangePredicate(genRelation.GenerateAttributes.First(a => a.Name == "Weight"), new ConstantVariable(0.3), new ConstantVariable(0.2));
            //IQueryDesc query = new QueryDesc(genRelation, new BooleanPredicate(left, right, BooleanComparisonType.OR));

            //IPredicate left = relationGen.GenerateAttributeConjunctionDisjunctionRangePredicate(genRelation.GenerateAttributes.First(a => a.Name == "Weight"), new ConstantVariable(0.1), new ConstantVariable(0.3), new ConstantVariable(0.4), new ConstantVariable(0.5));
            //IPredicate right = relationGen.GenerateAttributePredicate(genRelation.GenerateAttributes.First(a => a.Name == "Height"), new ConstantVariable(0.2));
            //IQueryDesc query = new QueryDesc(genRelation, new BooleanPredicate(left, right, BooleanComparisonType.OR));

            int match = 0;
            //foreach (ITuple tuple in genRelation.Execute(query))
            //{
            //    match++;
            //   // Console.WriteLine(tuple.ToString());
            //}

            //Console.WriteLine(string.Format("Query: {0}", query.ToString()));
            Console.WriteLine(string.Format("Matched {0} of {1} => {2}", match, total, (double)match / (double)total));
        }

        public static void TestQueryExecute()
        {
            RelationGenerator<test> relationGen = new RelationGenerator<test>();

            int total = 1000000;
            var s2 = Stopwatch.StartNew();
            IRelation<test> relation = relationGen.Generate(total);
            s2.Stop();

            //IPredicate predicate = new AttributePredicate(new AttributeDesc("Age", typeof(int)), ComparisonType.GREATER_THAN_OR_EQUAL, Comparer<object>.Create((left, right) => ((int)left).CompareTo((int)right)), () => 7);
            // IPredicate left = new IntegerAttributePredicate(new AttributeDesc("Age", typeof(int)), ComparisonType.GREATER_THAN_OR_EQUAL, 9);
            //IPredicate right = new IntegerAttributePredicate(new AttributeDesc("Weight", typeof(double)), ComparisonType.LESS_THAN, 175);
            //IPredicate predicate = new BooleanPredicate(left, right, BooleanComparisonType.AND);

            //IPredicate predicate = new DoubleAttributePredicate(new AttributeDesc("Weight", typeof(double)), ComparisonType.LESS_THAN, 175);
            //IPredicate predicate = new RandomDoubleAttributePredicate(new AttributeDesc("Weight", typeof(double)), ComparisonType.LESS_THAN, new UniformVariable(150, 175));
            //QueryDesc query = new QueryDesc(r, predicate);
            ComparableAttributePredicateGenerator<test> predGen = new ComparableAttributePredicateGenerator<test>();
            IPredicate<test> predicate = predGen.GenerateAttributePredicate(t => t.Weight, new ConstantVariable(0.1));
            IQuery<test, test> query = new SimpleQuery<test>(predicate);
            int match = 0;
            var s3 = Stopwatch.StartNew();
            foreach (test tuple in relation.Execute(query).Tuples)
            {
                match++;
                //Console.WriteLine(string.Format("Size: {0}", tuple.Attributes.Sum(a => a.SizeInBytes())));
                //Console.WriteLine(tuple.ToString());

            }
            s3.Stop();
            var s1 = Stopwatch.StartNew();
            int sizeInBytes = relation.GetSizeInBytes();
            s1.Stop();
            Console.WriteLine(string.Format("Size: {0} bytes, calculated in {1}ms", sizeInBytes, s1.ElapsedMilliseconds));
            Console.WriteLine(string.Format("Generated {0} in {1}ms", total, s2.ElapsedMilliseconds));
            Console.WriteLine(string.Format("Matched {0} of {1} in {2}ms", match, total, s3.ElapsedMilliseconds));
        }

        public static void TestQueryToString()
        {
            RelationGenerator<test> relationGen = new RelationGenerator<test>();
            ComparableAttributePredicateGenerator<test> predGen = new ComparableAttributePredicateGenerator<test>();
            IPredicate<test> predicate = predGen.GenerateAttributePredicate(t => t.Age, new ConstantVariable(0.10));
            SimpleQuery<test> query = new SimpleQuery<test>(predicate);

            
            Console.WriteLine(query.ToString());
        }

        public static void TestNormalDistribution()
        {
            IList<double> genValues = new List<double>();

            NormalVariable normal = new NormalVariable(8, 3);
            double size = 1000000;
            for (int ndx = 0; ndx < (int)size; ndx++)
            {
                genValues.Add(normal.Sample());
            }

            int countWithinOneStd = 0;
            foreach (double val in genValues)
            {
                double oneStd = Math.Sqrt(normal.Variance);
                if (val > (normal.Mean - oneStd) && val < (normal.Mean + oneStd)) countWithinOneStd++;
            }

            double percentWithin = countWithinOneStd / size;

            Console.WriteLine(percentWithin);
        }

        public static void TestNormalQuantile()
        {
            IList<double> genValues = new List<double>();

            NormalVariable normal = new NormalVariable(8, 4);
            double testProb = 0.8;
            double size = 1000000;
            for (int ndx = 0; ndx < (int)size; ndx++)
            {
                genValues.Add(normal.Sample());
            }

            double x = normal.CalculateValue(testProb);

            int countValuesLess = 0;
            foreach (double val in genValues)
            {
                if (val < x) countValuesLess++;
            }

            double percentLess = countValuesLess / size;

            Console.WriteLine(percentLess);
        }

        public static void TestUniformQuantile()
        {
            IList<double> genValues = new List<double>();

            UniformVariable normal = new UniformVariable(0, 100);
            double testProb = 0.8;
            double size = 1000000;
            for (int ndx = 0; ndx < (int)size; ndx++)
            {
                genValues.Add(normal.Sample());
            }

            double x = normal.CalculateValue(testProb);

            int countValuesLess = 0;
            foreach (double val in genValues)
            {
                if (val < x) countValuesLess++;
            }

            double percentLess = countValuesLess / size;

            Console.WriteLine(percentLess);
        }

        public static void TestTriangleQuantile()
        {
            IList<double> genValues = new List<double>();

            TriangularVariable normal = new TriangularVariable(0, 10, 3);
            double testProb = 0.8;
            double size = 1000000;
            for (int ndx = 0; ndx < (int)size; ndx++)
            {
                genValues.Add(normal.Sample());
            }

            double x = normal.CalculateValue(testProb);

            int countValuesLess = 0;
            foreach (double val in genValues)
            {
                if (val < x) countValuesLess++;
            }

            double percentLess = countValuesLess / size;

            Console.WriteLine(percentLess);
        }

        public static void TestNormalQuantileRange()
        {
            IList<double> genValues = new List<double>();

            NormalVariable normal = new NormalVariable(8, 4);
            double testProbA = 0.2;
            double testProbB = 0.5;
            double size = 1000000;
            for (int ndx = 0; ndx < (int)size; ndx++)
            {
                genValues.Add(normal.Sample());
            }

            double A = normal.CalculateValue(testProbA);
            double B = normal.CalculateValue(testProbB);


            int countValuesLess = 0;
            foreach (double val in genValues)
            {
                if (val > A && val < B) countValuesLess++;
            }

            double percentLess = countValuesLess / size;

            Console.WriteLine(percentLess);
        }

        public static void TestExpQuantile()
        {
            IList<double> genValues = new List<double>();

            ExponentialVariable normal = new ExponentialVariable(0.4);
            double testProb = 0.8;
            double size = 1000000;
            for (int ndx = 0; ndx < (int)size; ndx++)
            {
                genValues.Add(normal.Sample());
            }

            double x = normal.CalculateValue(testProb);

            int countValuesLess = 0;
            foreach (double val in genValues)
            {
                if (val < x) countValuesLess++;
            }

            double percentLess = countValuesLess / size;

            Console.WriteLine(percentLess);
        }

        public static void TestParetoQuantile()
        {
            IList<double> genValues = new List<double>();

            ParetoVariable normal = new ParetoVariable();
            double testProb = 0.7;
            double size = 1000000;
            for (int ndx = 0; ndx < (int)size; ndx++)
            {
                genValues.Add(normal.Sample());
            }

            double x = normal.CalculateValue(testProb);

            int countValuesLess = 0;
            foreach (double val in genValues)
            {
                if (val < x) countValuesLess++;
            }

            double percentLess = countValuesLess / size;

            Console.WriteLine(percentLess);
        }

        public static void TestWeibullQuantile()
        {
            IList<double> genValues = new List<double>();

            WeibullVariable normal = new WeibullVariable();
            double testProb = 0.8;
            double size = 1000000;
            for (int ndx = 0; ndx < (int)size; ndx++)
            {
                genValues.Add(normal.Sample());
            }

            double x = normal.CalculateValue(testProb);

            int countValuesLess = 0;
            foreach (double val in genValues)
            {
                if (val < x) countValuesLess++;
            }

            double percentLess = countValuesLess / size;

            Console.WriteLine(percentLess);
        }

        public static void TestLogNormalQuantile()
        {
            IList<double> genValues = new List<double>();

            LogNormalVariable normal = new LogNormalVariable();
            double testProb = 0.8;
            double size = 1000000;
            for (int ndx = 0; ndx < (int)size; ndx++)
            {
                genValues.Add(normal.Sample());
            }

            double x = normal.CalculateValue(testProb);

            int countValuesLess = 0;
            foreach (double val in genValues)
            {
                if (val < x) countValuesLess++;
            }

            double percentLess = countValuesLess / size;

            Console.WriteLine(percentLess);
        }

        public static void TestTriangleQuantileRange()
        {
            IList<double> genValues = new List<double>();

            TriangularVariable normal = new TriangularVariable(5, 50, 40);
            double testProbA = 0.2;
            double testProbB = 0.7;
            double size = 1000000;
            for (int ndx = 0; ndx < (int)size; ndx++)
            {
                genValues.Add(normal.Sample());
            }

            double A = normal.CalculateValue(testProbA);
            double B = normal.CalculateValue(testProbB);


            int countValuesLess = 0;
            foreach (double val in genValues)
            {
                if (val > A && val < B) countValuesLess++;
            }

            double percentLess = countValuesLess / size;

            Console.WriteLine(percentLess);
        }

        public static void TestRelationGeneration()
        {
            RelationGenerator<test> relationGen = new RelationGenerator<test>();
            IRelation<test> relation = relationGen.Generate(100);
            Console.WriteLine(relation.ToString());
        }

        public static void TestTypedRelationGeneration()
        {
            //GeneratedRelationDesc r = new TypedGeneratedRelationDesc<test>("test");

            ////RelationGenerator relationGen = new RelationGenerator(r, GeneratorResolver);

            //RelationGenerator relationGen = new RelationGenerator(r);

            //IGeneratedRelation genRelation = relationGen.Generate(100);

            //Console.Write(genRelation.ToString());
        }
    }

    public class IdSchema : RandomNumbericAttributeSchema
    {
        public IdSchema() : base(new SequentialVariable(0, double.MaxValue)) { }
    }

    public class NameSchema : RandomLowerAlphaStringAttributeSchema
    {
        public NameSchema() : base(new NormalVariable(8, 3)) { }
    }

    public class AgeSchema : RandomNumbericAttributeSchema
    {
        public AgeSchema() : base(new NormalVariable(30, 30)) { }
    }

    public class WeightSchema : RandomNumbericAttributeSchema
    {
        public WeightSchema() : base(new UniformVariable(100, 200)) { }
    }

    public class HeightSchema : RandomNumbericAttributeSchema
    {
        public HeightSchema() : base(new TriangularVariable(60, 84, 68)) { }
    }

    public class proj_test
    {
        public int Id { get; set; }
        public double Weight { get; set; }
    }

    [Tuple("tests")]
    public class test 
    {
        [RandomTuplePropertyAttribute(typeof(IdSchema))]
        public int Id { get; set; }

        [RandomTuplePropertyAttribute(typeof(NameSchema))]
        public string Name { get; set; }

        [RandomTuplePropertyAttribute(typeof(AgeSchema))]
        public int Age { get; set; }

        [RandomTuplePropertyAttribute(typeof(WeightSchema))]
        public double Weight { get; set; }

        [RandomTuplePropertyAttribute(typeof(HeightSchema))]
        public double Height { get; set; }
    }
}
