﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using OUDBCommonLibs.Interfaces;
using OUDBCommonLibs.Variables;
using OUDBCommonLibs.Attributes;
using OUDBCommonLibs.Relation;
using OUDBCommonLibs.Predicates;
using OUDBCommonLibs.IntervalTree;
using OUDBCommonLibs.Generators;
using OUDBCommonLibs.ExtensionMethods;
using OUDBQueryGeneration.QueryBatchGenerator;
using OUDBExperiments.Implementaions;


namespace OUDBQueryGenerationConsole
{
    class Program
    {
        static void Main(string[] args)
        {
            //GenerateRelationFile();
            GenerateQueryBatchFile();
            Console.ReadLine();
        }

        public static void GenerateRelationFile()
        {
            string fileName = @"allUniformIntegerRelation.tbl";


            RelationGenerator<ExpTable> relationGen = new RelationGenerator<ExpTable>();
            
            RelationWrapper<ExpTable> rItem = new RelationWrapper<ExpTable>(relationGen.Generate(200000));

            rItem.SaveToFile(fileName);
        }


        public static void GenerateQueryBatchFile()
        {
            string fileName = @"queryBatchIntegerRelation.txt";


            IPredicate<ExpTable> a = new ComparableAttributePredicate<ExpTable, int>(t => t.Id, ComparisonType.LESS_THAN_OR_EQUAL, 8857);
            
            double v = a.PercentOfRelation();

            Console.WriteLine("percentage: {0}", v);

            //IList<Interval<int>> intervals = a.ComputeIntervals<ExpTable, int>();
            //IPredicate<ExpTable> pred = intervals.ToPredicate<ExpTable, int>(t => t.IntAttr1);


            /*for (int i = 1; i < 10; ++i)
            {
                ICachedQueryBatchExperiment<ExpTable> exp = new CachedQueryBatchExperiment<ExpTable>(10, 200000, 100000, new UniformVariable(0.1, 0.2), (double)i/10.0, SemanticQueryType.PARTIAL, 0.1);
                IQueryBatchGenerator<ExpTable> gen = new SingleAttributeCachedQueryBatchGenerator<ExpTable, double>(exp, t => t.IntAttr1);

                IList<IQuery<ExpTable>> lst = gen.GenerateQueryBatch(50);

                Console.WriteLine(lst);

                //ComparableAttributePredicateGenerator<ExpTable> predGen = new ComparableAttributePredicateGenerator<ExpTable>();
                //IPredicate<ExpTable> predicate = predGen.GenerateAttributePredicate(t => t.IntAttr1, new ConstantVariable(0.1), false);

                //Console.WriteLine(predicate.ToString());
            }
            

            //IList<Interval<double>> list = predicate.ComputeIntervals<ExpTable, double>();

            //Console.WriteLine(string.Join(", ", list.Select(t => string.Format("[{0},{1}]", t.Start, t.End))));*/
        }



        public class IdSchema : RandomNumbericAttributeSchema
        {
            public IdSchema() : base(new SequentialVariable(1, int.MaxValue)) { }
        }

        public class NameSchema : RandomLowerAlphaStringAttributeSchema
        {
            public NameSchema() : base(new NormalVariable(15, 5)) { }
        }

        public class InfoSchema : RandomLowerAlphaStringAttributeSchema
        {
            public InfoSchema() : base(new NormalVariable(15, 5)) { }
        }

        public class IntAttr1Schema : RandomNumbericAttributeSchema
        {
            public IntAttr1Schema() : base(new UniformVariable(0, 10000)) { }
        }

        public class IntAttr2Schema : RandomNumbericAttributeSchema
        {
            public IntAttr2Schema() : base(new UniformVariable(0, 50000)) { }
        }

        public class IntAttr3Schema : RandomNumbericAttributeSchema
        {
            public IntAttr3Schema() : base(new UniformVariable(0, 100000)) { }
        }

        
        public class ExpTable: ISizeable
        {
            [RandomTuplePropertyAttribute(typeof(IdSchema))]
            public int Id { get; set; }

            [RandomTuplePropertyAttribute(typeof(NameSchema))]
            public string Name { get; set; }

            [RandomTuplePropertyAttribute(typeof(InfoSchema))]
            public string Info { get; set; }

            [RandomTuplePropertyAttribute(typeof(IntAttr1Schema))]
            public int IntAttr1 { get; set; }

            [RandomTuplePropertyAttribute(typeof(IntAttr2Schema))]
            public int IntAttr2 { get; set; }

            [RandomTuplePropertyAttribute(typeof(IntAttr3Schema))]
            public int IntAttr3 { get; set; }

            public int GetSizeInBytes()
            {
                return sizeof(double) * 4 + 20 + 20; //aproximated maximum size
            }
        }

    }


}
